from django import forms
from django.forms.formsets import DELETION_FIELD_NAME, BaseFormSet
from django.forms.models import (
    BaseModelFormSet,
    inlineformset_factory,
    modelform_factory,
    modelformset_factory,
)
from django.forms.utils import ErrorDict, ErrorList
from django.test import TestCase

from .models import (
    Host,
    Manager,
    Network,
    ProfileNetwork,
    Restaurant,
    User,
    UserPreferences,
    UserProfile,
    UserSite,
)


class InlineFormsetTests(TestCase):
    def test_formset_over_to_field(self):
        """
        A formset over a ForeignKey with a to_field can be saved.
        """
        Form = modelform_factory(User, fields="__all__")
        FormSet = inlineformset_factory(User, UserSite, fields="__all__")

        # Instantiate the Form and FormSet to prove
        # you can create a form with no data
        form = Form()
        form_set = FormSet(instance=User())

        # Now create a new User and UserSite instance
        data = {
            "serial": "1",
            "username": "apollo13",
            "usersite_set-TOTAL_FORMS": "1",
            "usersite_set-INITIAL_FORMS": "0",
            "usersite_set-MAX_NUM_FORMS": "0",
            "usersite_set-0-data": "10",
            "usersite_set-0-user": "apollo13",
        }
        user = User()
        form = Form(data)
        if form.is_valid():
            user = form.save()
        else:
            self.fail("Errors found on form:%s" % form_set)

        form_set = FormSet(data, instance=user)
        if form_set.is_valid():
            form_set.save()
            usersite = UserSite.objects.values()
            self.assertEqual(usersite[0]["data"], 10)
            self.assertEqual(usersite[0]["user_id"], "apollo13")
        else:
            self.fail("Errors found on formset:%s" % form_set.errors)

        # Now update the UserSite instance
        data = {
            "usersite_set-TOTAL_FORMS": "1",
            "usersite_set-INITIAL_FORMS": "1",
            "usersite_set-MAX_NUM_FORMS": "0",
            "usersite_set-0-id": str(usersite[0]["id"]),
            "usersite_set-0-data": "11",
            "usersite_set-0-user": "apollo13",
        }
        form_set = FormSet(data, instance=user)
        if form_set.is_valid():
            form_set.save()
            usersite = UserSite.objects.values()
            self.assertEqual(usersite[0]["data"], 11)
            self.assertEqual(usersite[0]["user_id"], "apollo13")
        else:
            self.fail("Errors found on formset:%s" % form_set.errors)

        # Now add a new UserSite instance
        data = {
            "usersite_set-TOTAL_FORMS": "2",
            "usersite_set-INITIAL_FORMS": "1",
            "usersite_set-MAX_NUM_FORMS": "0",
            "usersite_set-0-id": str(usersite[0]["id"]),
            "usersite_set-0-data": "11",
            "usersite_set-0-user": "apollo13",
            "usersite_set-1-data": "42",
            "usersite_set-1-user": "apollo13",
        }
        form_set = FormSet(data, instance=user)
        if form_set.is_valid():
            form_set.save()
            usersite = UserSite.objects.values().order_by("data")
            self.assertEqual(usersite[0]["data"], 11)
            self.assertEqual(usersite[0]["user_id"], "apollo13")
            self.assertEqual(usersite[1]["data"], 42)
            self.assertEqual(usersite[1]["user_id"], "apollo13")
        else:
            self.fail("Errors found on formset:%s" % form_set.errors)

    def test_formset_over_inherited_model(self):
        """
        A formset over a ForeignKey with a to_field can be saved.
        """
        Form = modelform_factory(Restaurant, fields="__all__")
        FormSet = inlineformset_factory(Restaurant, Manager, fields="__all__")

        # Instantiate the Form and FormSet to prove
        # you can create a form with no data
        form = Form()
        form_set = FormSet(instance=Restaurant())

        # Now create a new Restaurant and Manager instance
        data = {
            "name": "Guido's House of Pasta",
            "manager_set-TOTAL_FORMS": "1",
            "manager_set-INITIAL_FORMS": "0",
            "manager_set-MAX_NUM_FORMS": "0",
            "manager_set-0-name": "Guido Van Rossum",
        }
        restaurant = User()
        form = Form(data)
        if form.is_valid():
            restaurant = form.save()
        else:
            self.fail("Errors found on form:%s" % form_set)

        form_set = FormSet(data, instance=restaurant)
        if form_set.is_valid():
            form_set.save()
            manager = Manager.objects.values()
            self.assertEqual(manager[0]["name"], "Guido Van Rossum")
        else:
            self.fail("Errors found on formset:%s" % form_set.errors)

        # Now update the Manager instance
        data = {
            "manager_set-TOTAL_FORMS": "1",
            "manager_set-INITIAL_FORMS": "1",
            "manager_set-MAX_NUM_FORMS": "0",
            "manager_set-0-id": str(manager[0]["id"]),
            "manager_set-0-name": "Terry Gilliam",
        }
        form_set = FormSet(data, instance=restaurant)
        if form_set.is_valid():
            form_set.save()
            manager = Manager.objects.values()
            self.assertEqual(manager[0]["name"], "Terry Gilliam")
        else:
            self.fail("Errors found on formset:%s" % form_set.errors)

        # Now add a new Manager instance
        data = {
            "manager_set-TOTAL_FORMS": "2",
            "manager_set-INITIAL_FORMS": "1",
            "manager_set-MAX_NUM_FORMS": "0",
            "manager_set-0-id": str(manager[0]["id"]),
            "manager_set-0-name": "Terry Gilliam",
            "manager_set-1-name": "John Cleese",
        }
        form_set = FormSet(data, instance=restaurant)
        if form_set.is_valid():
            form_set.save()
            manager = Manager.objects.values().order_by("name")
            self.assertEqual(manager[0]["name"], "John Cleese")
            self.assertEqual(manager[1]["name"], "Terry Gilliam")
        else:
            self.fail("Errors found on formset:%s" % form_set.errors)

    def test_inline_model_with_to_field(self):
        """
        #13794 --- An inline model with a to_field of a formset with instance
        has working relations.
        """
        FormSet = inlineformset_factory(User, UserSite, exclude=("is_superuser",))

        user = User.objects.create(username="guido", serial=1337)
        UserSite.objects.create(user=user, data=10)
        formset = FormSet(instance=user)

        # Testing the inline model's relation
        self.assertEqual(formset[0].instance.user_id, "guido")

    def test_inline_model_with_primary_to_field(self):
        """An inline model with a OneToOneField with to_field & primary key."""
        FormSet = inlineformset_factory(
            User, UserPreferences, exclude=("is_superuser",)
        )
        user = User.objects.create(username="guido", serial=1337)
        UserPreferences.objects.create(user=user, favorite_number=10)
        formset = FormSet(instance=user)
        self.assertEqual(formset[0].fields["user"].initial, "guido")

    def test_inline_model_with_to_field_to_rel(self):
        """
        #13794 --- An inline model with a to_field to a related field of a
        formset with instance has working relations.
        """
        FormSet = inlineformset_factory(UserProfile, ProfileNetwork, exclude=[])

        user = User.objects.create(username="guido", serial=1337, pk=1)
        self.assertEqual(user.pk, 1)
        profile = UserProfile.objects.create(user=user, about="about", pk=2)
        self.assertEqual(profile.pk, 2)
        ProfileNetwork.objects.create(profile=profile, network=10, identifier=10)
        formset = FormSet(instance=profile)

        # Testing the inline model's relation
        self.assertEqual(formset[0].instance.profile_id, 1)

    def test_formset_with_none_instance(self):
        "A formset with instance=None can be created. Regression for #11872"
        Form = modelform_factory(User, fields="__all__")
        FormSet = inlineformset_factory(User, UserSite, fields="__all__")

        # Instantiate the Form and FormSet to prove
        # you can create a formset with an instance of None
        Form(instance=None)
        FormSet(instance=None)

    def test_empty_fields_on_modelformset(self):
        """
        No fields passed to modelformset_factory() should result in no fields
        on returned forms except for the id (#14119).
        """
        UserFormSet = modelformset_factory(User, fields=())
        formset = UserFormSet()
        for form in formset.forms:
            self.assertIn("id", form.fields)
            self.assertEqual(len(form.fields), 1)

    def test_save_as_new_with_new_inlines(self):
        """
        Existing and new inlines are saved with save_as_new.

        Regression for #14938.
        """
        efnet = Network.objects.create(name="EFNet")
        host1 = Host.objects.create(hostname="irc.he.net", network=efnet)

        HostFormSet = inlineformset_factory(Network, Host, fields="__all__")

        # Add a new host, modify previous host, and save-as-new
        data = {
            "host_set-TOTAL_FORMS": "2",
            "host_set-INITIAL_FORMS": "1",
            "host_set-MAX_NUM_FORMS": "0",
            "host_set-0-id": str(host1.id),
            "host_set-0-hostname": "tranquility.hub.dal.net",
            "host_set-1-hostname": "matrix.de.eu.dal.net",
        }

        # To save a formset as new, it needs a new hub instance
        dalnet = Network.objects.create(name="DALnet")
        formset = HostFormSet(data, instance=dalnet, save_as_new=True)

        self.assertTrue(formset.is_valid())
        formset.save()
        self.assertQuerySetEqual(
            dalnet.host_set.order_by("hostname"),
            Host.objects.filter(
                hostname__in=[
                    "matrix.de.eu.dal.net",
                    "tranquility.hub.dal.net",
                ]
            ).order_by("hostname"),
        )

    def test_initial_data(self):
        user = User.objects.create(username="bibi", serial=1)
        UserSite.objects.create(user=user, data=7)
        FormSet = inlineformset_factory(User, UserSite, extra=2, fields="__all__")

        formset = FormSet(instance=user, initial=[{"data": 41}, {"data": 42}])
        self.assertEqual(formset.forms[0].initial["data"], 7)
        self.assertEqual(formset.extra_forms[0].initial["data"], 41)
        self.assertIn('value="42"', formset.extra_forms[1].as_p())


class FormsetTests(TestCase):
    def test_error_class(self):
        """
        Test the type of Formset and Form error attributes
        """
        Formset = modelformset_factory(User, fields="__all__")
        data = {
            "form-TOTAL_FORMS": "2",
            "form-INITIAL_FORMS": "0",
            "form-MAX_NUM_FORMS": "0",
            "form-0-id": "",
            "form-0-username": "apollo13",
            "form-0-serial": "1",
            "form-1-id": "",
            "form-1-username": "apollo13",
            "form-1-serial": "2",
        }
        formset = Formset(data)
        # check if the returned error classes are correct
        # note: formset.errors returns a list as documented
        self.assertIsInstance(formset.errors, list)
        self.assertIsInstance(formset.non_form_errors(), ErrorList)
        for form in formset.forms:
            self.assertIsInstance(form.errors, ErrorDict)
            self.assertIsInstance(form.non_field_errors(), ErrorList)

    def test_initial_data(self):
        User.objects.create(username="bibi", serial=1)
        Formset = modelformset_factory(User, fields="__all__", extra=2)
        formset = Formset(initial=[{"username": "apollo11"}, {"username": "apollo12"}])
        self.assertEqual(formset.forms[0].initial["username"], "bibi")
        self.assertEqual(formset.extra_forms[0].initial["username"], "apollo11")
        self.assertIn('value="apollo12"', formset.extra_forms[1].as_p())

    def test_extraneous_query_is_not_run(self):
        Formset = modelformset_factory(Network, fields="__all__")
        data = {
            "test-TOTAL_FORMS": "1",
            "test-INITIAL_FORMS": "0",
            "test-MAX_NUM_FORMS": "",
            "test-0-name": "Random Place",
        }
        with self.assertNumQueries(1):
            formset = Formset(data, prefix="test")
            formset.save()


class CustomWidget(forms.widgets.TextInput):
    pass


class UserSiteForm(forms.ModelForm):
    class Meta:
        model = UserSite
        fields = "__all__"
        widgets = {
            "id": CustomWidget,
            "data": CustomWidget,
        }
        localized_fields = ("data",)


class Callback:
    def __init__(self):
        self.log = []

    def __call__(self, db_field, **kwargs):
        self.log.append((db_field, kwargs))
        return db_field.formfield(**kwargs)


class FormfieldCallbackTests(TestCase):
    """
    Regression for #13095 and #17683: Using base forms with widgets
    defined in Meta should not raise errors and BaseModelForm should respect
    the specified pk widget.
    """

    def test_inlineformset_factory_default(self):
        Formset = inlineformset_factory(
            User, UserSite, form=UserSiteForm, fields="__all__"
        )
        form = Formset().forms[0]
        self.assertIsInstance(form["id"].field.widget, CustomWidget)
        self.assertIsInstance(form["data"].field.widget, CustomWidget)
        self.assertFalse(form.fields["id"].localize)
        self.assertTrue(form.fields["data"].localize)

    def test_modelformset_factory_default(self):
        Formset = modelformset_factory(UserSite, form=UserSiteForm)
        form = Formset().forms[0]
        self.assertIsInstance(form["id"].field.widget, CustomWidget)
        self.assertIsInstance(form["data"].field.widget, CustomWidget)
        self.assertFalse(form.fields["id"].localize)
        self.assertTrue(form.fields["data"].localize)

    def assertCallbackCalled(self, callback):
        id_field, user_field, data_field = UserSite._meta.fields
        expected_log = [
            (id_field, {"widget": CustomWidget}),
            (user_field, {}),
            (data_field, {"widget": CustomWidget, "localize": True}),
        ]
        self.assertEqual(callback.log, expected_log)

    def test_inlineformset_custom_callback(self):
        callback = Callback()
        inlineformset_factory(
            User,
            UserSite,
            form=UserSiteForm,
            formfield_callback=callback,
            fields="__all__",
        )
        self.assertCallbackCalled(callback)

    def test_modelformset_custom_callback(self):
        callback = Callback()
        modelformset_factory(UserSite, form=UserSiteForm, formfield_callback=callback)
        self.assertCallbackCalled(callback)


class BaseCustomDeleteFormSet(BaseFormSet):
    """
    A formset mix-in that lets a form decide if it's to be deleted.
    Works for BaseFormSets. Also works for ModelFormSets with #14099 fixed.

    form.should_delete() is called. The formset delete field is also
    suppressed.
    """

    def add_fields(self, form, index):
        super().add_fields(form, index)
        self.can_delete = True
        if DELETION_FIELD_NAME in form.fields:
            del form.fields[DELETION_FIELD_NAME]

    def _should_delete_form(self, form):
        return hasattr(form, "should_delete") and form.should_delete()


class FormfieldShouldDeleteFormTests(TestCase):
    """
    BaseModelFormSet should use ModelFormSet method _should_delete_form.
    """

    class BaseCustomDeleteModelFormSet(BaseModelFormSet, BaseCustomDeleteFormSet):
        """Model FormSet with CustomDelete MixIn"""

    class CustomDeleteUserForm(forms.ModelForm):
        """A model form with a 'should_delete' method"""

        class Meta:
            model = User
            fields = "__all__"

        def should_delete(self):
            """Delete form if odd serial."""
            return self.instance.serial % 2 != 0

    NormalFormset = modelformset_factory(
        User, form=CustomDeleteUserForm, can_delete=True
    )
    DeleteFormset = modelformset_factory(
        User, form=CustomDeleteUserForm, formset=BaseCustomDeleteModelFormSet
    )

    data = {
        "form-TOTAL_FORMS": "4",
        "form-INITIAL_FORMS": "0",
        "form-MAX_NUM_FORMS": "4",
        "form-0-username": "John",
        "form-0-serial": "1",
        "form-1-username": "Paul",
        "form-1-serial": "2",
        "form-2-username": "George",
        "form-2-serial": "3",
        "form-3-username": "Ringo",
        "form-3-serial": "5",
    }

    delete_all_ids = {
        "form-0-DELETE": "1",
        "form-1-DELETE": "1",
        "form-2-DELETE": "1",
        "form-3-DELETE": "1",
    }

    def test_init_database(self):
        """Add test data to database via formset"""
        formset = self.NormalFormset(self.data)
        self.assertTrue(formset.is_valid())
        self.assertEqual(len(formset.save()), 4)

    def test_no_delete(self):
        """Verify base formset doesn't modify database"""
        # reload database
        self.test_init_database()

        # pass standard data dict & see none updated
        data = dict(self.data)
        data["form-INITIAL_FORMS"] = 4
        data.update(
            {
                "form-%d-id" % i: user.pk
                for i, user in enumerate(User.objects.order_by("pk"))
            }
        )
        formset = self.NormalFormset(data, queryset=User.objects.all())
        self.assertTrue(formset.is_valid())
        self.assertEqual(len(formset.save()), 0)
        self.assertEqual(len(User.objects.all()), 4)

    def test_all_delete(self):
        """Verify base formset honors DELETE field"""
        # reload database
        self.test_init_database()

        # create data dict with all fields marked for deletion
        data = dict(self.data)
        data["form-INITIAL_FORMS"] = 4
        data.update(
            {"form-%d-id" % i: user.pk for i, user in enumerate(User.objects.all())}
        )
        data.update(self.delete_all_ids)
        formset = self.NormalFormset(data, queryset=User.objects.all())
        self.assertTrue(formset.is_valid())
        self.assertEqual(len(formset.save()), 0)
        self.assertEqual(len(User.objects.all()), 0)

    def test_custom_delete(self):
        """Verify DeleteFormset ignores DELETE field and uses form method"""
        # reload database
        self.test_init_database()

        # Create formset with custom Delete function
        # create data dict with all fields marked for deletion
        data = dict(self.data)
        data["form-INITIAL_FORMS"] = 4
        data.update(
            {
                "form-%d-id" % i: user.pk
                for i, user in enumerate(User.objects.order_by("pk"))
            }
        )
        data.update(self.delete_all_ids)
        formset = self.DeleteFormset(data, queryset=User.objects.all())

        # Three with odd serial values were deleted.
        self.assertTrue(formset.is_valid())
        self.assertEqual(len(formset.save()), 0)
        self.assertEqual(User.objects.count(), 1)

        # No odd serial values left.
        odd_serials = [user.serial for user in User.objects.all() if user.serial % 2]
        self.assertEqual(len(odd_serials), 0)


class RedeleteTests(TestCase):
    def test_resubmit(self):
        u = User.objects.create(username="foo", serial=1)
        us = UserSite.objects.create(user=u, data=7)
        formset_cls = inlineformset_factory(User, UserSite, fields="__all__")
        data = {
            "serial": "1",
            "username": "foo",
            "usersite_set-TOTAL_FORMS": "1",
            "usersite_set-INITIAL_FORMS": "1",
            "usersite_set-MAX_NUM_FORMS": "1",
            "usersite_set-0-id": str(us.pk),
            "usersite_set-0-data": "7",
            "usersite_set-0-user": "foo",
            "usersite_set-0-DELETE": "1",
        }
        formset = formset_cls(data, instance=u)
        self.assertTrue(formset.is_valid())
        formset.save()
        self.assertEqual(UserSite.objects.count(), 0)
        formset = formset_cls(data, instance=u)
        # Even if the "us" object isn't in the DB any more, the form
        # validates.
        self.assertTrue(formset.is_valid())
        formset.save()
        self.assertEqual(UserSite.objects.count(), 0)

    def test_delete_already_deleted(self):
        u = User.objects.create(username="foo", serial=1)
        us = UserSite.objects.create(user=u, data=7)
        formset_cls = inlineformset_factory(User, UserSite, fields="__all__")
        data = {
            "serial": "1",
            "username": "foo",
            "usersite_set-TOTAL_FORMS": "1",
            "usersite_set-INITIAL_FORMS": "1",
            "usersite_set-MAX_NUM_FORMS": "1",
            "usersite_set-0-id": str(us.pk),
            "usersite_set-0-data": "7",
            "usersite_set-0-user": "foo",
            "usersite_set-0-DELETE": "1",
        }
        formset = formset_cls(data, instance=u)
        us.delete()
        self.assertTrue(formset.is_valid())
        formset.save()
        self.assertEqual(UserSite.objects.count(), 0)



import datetime

from django.db import models


class Country(models.Model):
    # Table Column Fields
    name = models.CharField(max_length=50)

    def __str__(self):
        return self.name


class Person(models.Model):
    # Table Column Fields
    name = models.CharField(max_length=128)
    person_country_id = models.IntegerField()

    # Relation Fields
    person_country = models.ForeignObject(
        Country,
        from_fields=["person_country_id"],
        to_fields=["id"],
        on_delete=models.CASCADE,
    )
    friends = models.ManyToManyField("self", through="Friendship", symmetrical=False)

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return self.name


class Group(models.Model):
    # Table Column Fields
    name = models.CharField(max_length=128)
    group_country = models.ForeignKey(Country, models.CASCADE)
    members = models.ManyToManyField(
        Person, related_name="groups", through="Membership"
    )

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return self.name


class Membership(models.Model):
    # Table Column Fields
    membership_country = models.ForeignKey(Country, models.CASCADE, null=True)
    date_joined = models.DateTimeField(default=datetime.datetime.now)
    invite_reason = models.CharField(max_length=64, null=True)
    person_id = models.IntegerField()
    group_id = models.IntegerField(blank=True, null=True)

    # Relation Fields
    person = models.ForeignObject(
        Person,
        from_fields=["person_id", "membership_country"],
        to_fields=["id", "person_country_id"],
        on_delete=models.CASCADE,
    )
    group = models.ForeignObject(
        Group,
        from_fields=["group_id", "membership_country"],
        to_fields=["id", "group_country"],
        on_delete=models.CASCADE,
    )

    class Meta:
        ordering = ("date_joined", "invite_reason")

    def __str__(self):
        group_name = self.group.name if self.group_id else "NULL"
        return "%s is a member of %s" % (self.person.name, group_name)


class Friendship(models.Model):
    # Table Column Fields
    from_friend_country = models.ForeignKey(
        Country, models.CASCADE, related_name="from_friend_country"
    )
    from_friend_id = models.IntegerField()
    to_friend_country_id = models.IntegerField()
    to_friend_id = models.IntegerField(null=True)

    # Relation Fields
    from_friend = models.ForeignObject(
        Person,
        on_delete=models.CASCADE,
        from_fields=["from_friend_country", "from_friend_id"],
        to_fields=["person_country_id", "id"],
        related_name="from_friend",
    )

    to_friend_country = models.ForeignObject(
        Country,
        from_fields=["to_friend_country_id"],
        to_fields=["id"],
        related_name="to_friend_country",
        on_delete=models.CASCADE,
    )

    to_friend = models.ForeignObject(
        Person,
        from_fields=["to_friend_country_id", "to_friend_id"],
        to_fields=["person_country_id", "id"],
        related_name="to_friend+",
        on_delete=models.CASCADE,
    )



# Copyright (c) 2008-2009 Aryeh Leib Taurog, all rights reserved.
# Modified from original contribution by Aryeh Leib Taurog, which was
# released under the New BSD license.

from django.contrib.gis.geos import (
    LinearRing,
    LineString,
    MultiPoint,
    Point,
    Polygon,
    fromstr,
)
from django.test import SimpleTestCase


def api_get_distance(x):
    return x.distance(Point(-200, -200))


def api_get_buffer(x):
    return x.buffer(10)


def api_get_geom_typeid(x):
    return x.geom_typeid


def api_get_num_coords(x):
    return x.num_coords


def api_get_centroid(x):
    return x.centroid


def api_get_empty(x):
    return x.empty


def api_get_valid(x):
    return x.valid


def api_get_simple(x):
    return x.simple


def api_get_ring(x):
    return x.ring


def api_get_boundary(x):
    return x.boundary


def api_get_convex_hull(x):
    return x.convex_hull


def api_get_extent(x):
    return x.extent


def api_get_area(x):
    return x.area


def api_get_length(x):
    return x.length


geos_function_tests = [
    val
    for name, val in vars().items()
    if hasattr(val, "__call__") and name.startswith("api_get_")
]


class GEOSMutationTest(SimpleTestCase):
    """
    Tests Pythonic Mutability of Python GEOS geometry wrappers
    get/set/delitem on a slice, normal list methods
    """

    def test00_GEOSIndexException(self):
        "Testing Geometry IndexError"
        p = Point(1, 2)
        for i in range(-2, 2):
            with self.subTest(i=i):
                p._checkindex(i)

        for i in (2, -3):
            with (
                self.subTest(i=i),
                self.assertRaisesMessage(IndexError, f"invalid index: {i}"),
            ):
                p._checkindex(i)

    def test01_PointMutations(self):
        "Testing Point mutations"
        for p in (Point(1, 2, 3), fromstr("POINT (1 2 3)")):
            with self.subTest(p=p):
                self.assertEqual(
                    p._get_single_external(1), 2.0, "Point _get_single_external"
                )

                # _set_single
                p._set_single(0, 100)
                self.assertEqual(p.coords, (100.0, 2.0, 3.0), "Point _set_single")

                # _set_list
                p._set_list(2, (50, 3141))
                self.assertEqual(p.coords, (50.0, 3141.0), "Point _set_list")

    def test02_PointExceptions(self):
        "Testing Point exceptions"
        msg = "Invalid parameters given for Point initialization."
        for i in (range(1), range(4)):
            with (
                self.subTest(i=i),
                self.assertRaisesMessage(TypeError, msg),
            ):
                Point(i)

    def test03_PointApi(self):
        "Testing Point API"
        q = Point(4, 5, 3)
        for p in (Point(1, 2, 3), fromstr("POINT (1 2 3)")):
            p[0:2] = [4, 5]
            for f in geos_function_tests:
                with self.subTest(p=p, f=f):
                    self.assertEqual(f(q), f(p), "Point " + f.__name__)

    def test04_LineStringMutations(self):
        "Testing LineString mutations"
        for ls in (
            LineString((1, 0), (4, 1), (6, -1)),
            fromstr("LINESTRING (1 0,4 1,6 -1)"),
        ):
            with self.subTest(ls=ls):
                self.assertEqual(
                    ls._get_single_external(1),
                    (4.0, 1.0),
                    "LineString _get_single_external",
                )

                # _set_single
                ls._set_single(0, (-50, 25))
                self.assertEqual(
                    ls.coords,
                    ((-50.0, 25.0), (4.0, 1.0), (6.0, -1.0)),
                    "LineString _set_single",
                )

                # _set_list
                ls._set_list(2, ((-50.0, 25.0), (6.0, -1.0)))
                self.assertEqual(
                    ls.coords, ((-50.0, 25.0), (6.0, -1.0)), "LineString _set_list"
                )

                lsa = LineString(ls.coords)
                for f in geos_function_tests:
                    with self.subTest(f=f):
                        self.assertEqual(f(lsa), f(ls), "LineString " + f.__name__)

    def test05_Polygon(self):
        "Testing Polygon mutations"
        for pg in (
            Polygon(
                ((1, 0), (4, 1), (6, -1), (8, 10), (1, 0)),
                ((5, 4), (6, 4), (6, 3), (5, 4)),
            ),
            fromstr("POLYGON ((1 0,4 1,6 -1,8 10,1 0),(5 4,6 4,6 3,5 4))"),
        ):
            with self.subTest(pg=pg):
                self.assertEqual(
                    pg._get_single_external(0),
                    LinearRing((1, 0), (4, 1), (6, -1), (8, 10), (1, 0)),
                    "Polygon _get_single_external(0)",
                )
                self.assertEqual(
                    pg._get_single_external(1),
                    LinearRing((5, 4), (6, 4), (6, 3), (5, 4)),
                    "Polygon _get_single_external(1)",
                )

                # _set_list
                pg._set_list(
                    2,
                    (
                        ((1, 2), (10, 0), (12, 9), (-1, 15), (1, 2)),
                        ((4, 2), (5, 2), (5, 3), (4, 2)),
                    ),
                )
                self.assertEqual(
                    pg.coords,
                    (
                        (
                            (1.0, 2.0),
                            (10.0, 0.0),
                            (12.0, 9.0),
                            (-1.0, 15.0),
                            (1.0, 2.0),
                        ),
                        ((4.0, 2.0), (5.0, 2.0), (5.0, 3.0), (4.0, 2.0)),
                    ),
                    "Polygon _set_list",
                )

                lsa = Polygon(*pg.coords)
                for f in geos_function_tests:
                    with self.subTest(f=f):
                        self.assertEqual(f(lsa), f(pg), "Polygon " + f.__name__)

    def test06_Collection(self):
        "Testing Collection mutations"
        points = (
            MultiPoint(*map(Point, ((3, 4), (-1, 2), (5, -4), (2, 8)))),
            fromstr("MULTIPOINT (3 4,-1 2,5 -4,2 8)"),
        )
        for mp in points:
            with self.subTest(mp=mp):
                self.assertEqual(
                    mp._get_single_external(2),
                    Point(5, -4),
                    "Collection _get_single_external",
                )

                mp._set_list(3, map(Point, ((5, 5), (3, -2), (8, 1))))
                self.assertEqual(
                    mp.coords,
                    ((5.0, 5.0), (3.0, -2.0), (8.0, 1.0)),
                    "Collection _set_list",
                )

                lsa = MultiPoint(*map(Point, ((5, 5), (3, -2), (8, 1))))
                for f in geos_function_tests:
                    with self.subTest(f=f):
                        self.assertEqual(f(lsa), f(mp), "MultiPoint " + f.__name__)



from dataclasses import dataclass, field, replace
from datetime import datetime
from inspect import isclass, iscoroutinefunction
from typing import Any, Callable, Dict, Optional

from asgiref.sync import async_to_sync, sync_to_async

from django.db.models.enums import TextChoices
from django.utils.json import normalize_json
from django.utils.module_loading import import_string
from django.utils.translation import pgettext_lazy

from .exceptions import TaskResultMismatch

DEFAULT_TASK_BACKEND_ALIAS = "default"
DEFAULT_TASK_PRIORITY = 0
DEFAULT_TASK_QUEUE_NAME = "default"
TASK_MAX_PRIORITY = 100
TASK_MIN_PRIORITY = -100
TASK_REFRESH_ATTRS = {
    "errors",
    "_return_value",
    "finished_at",
    "started_at",
    "last_attempted_at",
    "status",
    "enqueued_at",
    "worker_ids",
}


class TaskResultStatus(TextChoices):
    # The Task has just been enqueued, or is ready to be executed again.
    READY = ("READY", pgettext_lazy("Task", "Ready"))
    # The Task is currently running.
    RUNNING = ("RUNNING", pgettext_lazy("Task", "Running"))
    # The Task raised an exception during execution, or was unable to start.
    FAILED = ("FAILED", pgettext_lazy("Task", "Failed"))
    # The Task has finished running successfully.
    SUCCESSFUL = ("SUCCESSFUL", pgettext_lazy("Task", "Successful"))


@dataclass(frozen=True, slots=True, kw_only=True)
class Task:
    priority: int
    func: Callable  # The Task function.
    backend: str
    queue_name: str
    run_after: Optional[datetime]  # The earliest this Task will run.

    # Whether the Task receives the Task context when executed.
    takes_context: bool = False

    def __post_init__(self):
        self.get_backend().validate_task(self)

    @property
    def name(self):
        return self.func.__name__

    def using(
        self,
        *,
        priority=None,
        queue_name=None,
        run_after=None,
        backend=None,
    ):
        """Create a new Task with modified defaults."""

        changes = {}
        if priority is not None:
            changes["priority"] = priority
        if queue_name is not None:
            changes["queue_name"] = queue_name
        if run_after is not None:
            changes["run_after"] = run_after
        if backend is not None:
            changes["backend"] = backend
        return replace(self, **changes)

    def enqueue(self, *args, **kwargs):
        """Queue up the Task to be executed."""
        return self.get_backend().enqueue(self, args, kwargs)

    async def aenqueue(self, *args, **kwargs):
        """Queue up the Task to be executed."""
        return await self.get_backend().aenqueue(self, args, kwargs)

    def get_result(self, result_id):
        """
        Retrieve a task result by id.

        Raise TaskResultDoesNotExist if such result does not exist, or raise
        TaskResultMismatch if the result exists but belongs to another Task.
        """
        result = self.get_backend().get_result(result_id)
        if result.task.func != self.func:
            raise TaskResultMismatch(
                f"Task does not match (received {result.task.module_path!r})"
            )
        return result

    async def aget_result(self, result_id):
        """See get_result()."""
        result = await self.get_backend().aget_result(result_id)
        if result.task.func != self.func:
            raise TaskResultMismatch(
                f"Task does not match (received {result.task.module_path!r})"
            )
        return result

    def call(self, *args, **kwargs):
        if iscoroutinefunction(self.func):
            return async_to_sync(self.func)(*args, **kwargs)
        return self.func(*args, **kwargs)

    async def acall(self, *args, **kwargs):
        if iscoroutinefunction(self.func):
            return await self.func(*args, **kwargs)
        return await sync_to_async(self.func)(*args, **kwargs)

    def get_backend(self):
        from . import task_backends

        return task_backends[self.backend]

    @property
    def module_path(self):
        return f"{self.func.__module__}.{self.func.__qualname__}"


def task(
    function=None,
    *,
    priority=DEFAULT_TASK_PRIORITY,
    queue_name=DEFAULT_TASK_QUEUE_NAME,
    backend=DEFAULT_TASK_BACKEND_ALIAS,
    takes_context=False,
):
    from . import task_backends

    def wrapper(f):
        return task_backends[backend].task_class(
            priority=priority,
            func=f,
            queue_name=queue_name,
            backend=backend,
            takes_context=takes_context,
            run_after=None,
        )

    if function:
        return wrapper(function)
    return wrapper


@dataclass(frozen=True, slots=True, kw_only=True)
class TaskError:
    exception_class_path: str
    traceback: str

    @property
    def exception_class(self):
        # Lazy resolve the exception class.
        exception_class = import_string(self.exception_class_path)

        if not isclass(exception_class) or not issubclass(
            exception_class, BaseException
        ):
            raise ValueError(
                f"{self.exception_class_path!r} does not reference a valid exception."
            )
        return exception_class


@dataclass(frozen=True, slots=True, kw_only=True)
class TaskResult:
    task: Task

    id: str  # Unique identifier for the task result.
    status: TaskResultStatus
    enqueued_at: Optional[datetime]  # Time the task was enqueued.
    started_at: Optional[datetime]  # Time the task was started.
    finished_at: Optional[datetime]  # Time the task was finished.

    # Time the task was last attempted to be run.
    last_attempted_at: Optional[datetime]

    args: list  # Arguments to pass to the task function.
    kwargs: Dict[str, Any]  # Keyword arguments to pass to the task function.
    backend: str
    errors: list[TaskError]  # Errors raised when running the task.
    worker_ids: list[str]  # Workers which have processed the task.

    _return_value: Optional[Any] = field(init=False, default=None)

    def __post_init__(self):
        object.__setattr__(self, "args", normalize_json(self.args))
        object.__setattr__(self, "kwargs", normalize_json(self.kwargs))

    @property
    def return_value(self):
        """
        The return value of the task.

        If the task didn't succeed, an exception is raised.
        This is to distinguish against the task returning None.
        """
        if self.status == TaskResultStatus.SUCCESSFUL:
            return self._return_value
        elif self.status == TaskResultStatus.FAILED:
            raise ValueError("Task failed")
        else:
            raise ValueError("Task has not finished yet")

    @property
    def is_finished(self):
        return self.status in {TaskResultStatus.FAILED, TaskResultStatus.SUCCESSFUL}

    @property
    def attempts(self):
        return len(self.worker_ids)

    def refresh(self):
        """Reload the cached task data from the task store."""
        refreshed_task = self.task.get_backend().get_result(self.id)

        for attr in TASK_REFRESH_ATTRS:
            object.__setattr__(self, attr, getattr(refreshed_task, attr))

    async def arefresh(self):
        """
        Reload the cached task data from the task store
        """
        refreshed_task = await self.task.get_backend().aget_result(self.id)
        for attr in TASK_REFRESH_ATTRS:
            object.__setattr__(self, attr, getattr(refreshed_task, attr))


@dataclass(frozen=True, slots=True, kw_only=True)
class TaskContext:
    task_result: TaskResult

    @property
    def attempt(self):
        return self.task_result.attempts



import pickle
import sys
import unittest
from unittest.case import TestCase
from unittest.result import TestResult
from unittest.suite import TestSuite, _ErrorHolder

from django.test import SimpleTestCase
from django.test.runner import ParallelTestSuite, RemoteTestResult

from . import models

try:
    import tblib.pickling_support
except ImportError:
    tblib = None


def _test_error_exc_info():
    try:
        raise ValueError("woops")
    except ValueError:
        return sys.exc_info()


class ExceptionThatFailsUnpickling(Exception):
    """
    After pickling, this class fails unpickling with an error about incorrect
    arguments passed to __init__().
    """

    def __init__(self, arg):
        super().__init__()

    def __reduce__(self):
        # tblib 3.2+ makes exception subclasses picklable by default.
        # Return (cls, ()) so the constructor fails on unpickle, preserving
        # the needed behavior for test_pickle_errors_detection.
        return (self.__class__, ())


class ParallelTestRunnerTest(SimpleTestCase):
    """
    End-to-end tests of the parallel test runner.

    These tests are only meaningful when running tests in parallel using
    the --parallel option, though it doesn't hurt to run them not in
    parallel.
    """

    def test_subtest(self):
        """
        Passing subtests work.
        """
        for i in range(2):
            with self.subTest(index=i):
                self.assertEqual(i, i)

    def test_system_checks(self):
        self.assertEqual(models.Person.system_check_run_count, 1)


class SampleFailingSubtest(SimpleTestCase):
    # This method name doesn't begin with "test" to prevent test discovery
    # from seeing it.
    def dummy_test(self):
        """
        A dummy test for testing subTest failures.
        """
        for i in range(3):
            with self.subTest(index=i):
                self.assertEqual(i, 1)

    # This method name doesn't begin with "test" to prevent test discovery
    # from seeing it.
    def pickle_error_test(self):
        with self.subTest("TypeError: cannot pickle memoryview object"):
            self.x = memoryview(b"")
            self.fail("expected failure")


class SampleErrorTest(SimpleTestCase):
    @classmethod
    def setUpClass(cls):
        raise ValueError("woops")
        super().setUpClass()

    # This method name doesn't begin with "test" to prevent test discovery
    # from seeing it.
    def dummy_test(self):
        raise AssertionError("SampleErrorTest.dummy_test() was called")


class RemoteTestResultTest(SimpleTestCase):
    def test_was_successful_no_events(self):
        result = RemoteTestResult()
        self.assertIs(result.wasSuccessful(), True)

    def test_was_successful_one_success(self):
        result = RemoteTestResult()
        test = None
        result.startTest(test)
        try:
            result.addSuccess(test)
        finally:
            result.stopTest(test)
        self.assertIs(result.wasSuccessful(), True)

    def test_was_successful_one_expected_failure(self):
        result = RemoteTestResult()
        test = None
        result.startTest(test)
        try:
            result.addExpectedFailure(test, _test_error_exc_info())
        finally:
            result.stopTest(test)
        self.assertIs(result.wasSuccessful(), True)

    def test_was_successful_one_skip(self):
        result = RemoteTestResult()
        test = None
        result.startTest(test)
        try:
            result.addSkip(test, "Skipped")
        finally:
            result.stopTest(test)
        self.assertIs(result.wasSuccessful(), True)

    @unittest.skipUnless(tblib is not None, "requires tblib to be installed")
    def test_was_successful_one_error(self):
        result = RemoteTestResult()
        test = None
        result.startTest(test)
        try:
            result.addError(test, _test_error_exc_info())
        finally:
            result.stopTest(test)
        self.assertIs(result.wasSuccessful(), False)

    @unittest.skipUnless(tblib is not None, "requires tblib to be installed")
    def test_was_successful_one_failure(self):
        result = RemoteTestResult()
        test = None
        result.startTest(test)
        try:
            result.addFailure(test, _test_error_exc_info())
        finally:
            result.stopTest(test)
        self.assertIs(result.wasSuccessful(), False)

    @unittest.skipUnless(tblib is not None, "requires tblib to be installed")
    def test_add_error_before_first_test(self):
        result = RemoteTestResult()
        test_id = "test_foo (tests.test_foo.FooTest.test_foo)"
        test = _ErrorHolder(test_id)
        # Call addError() without a call to startTest().
        result.addError(test, _test_error_exc_info())

        (event,) = result.events
        self.assertEqual(event[0], "addError")
        self.assertEqual(event[1], -1)
        self.assertEqual(event[2], test_id)
        error_type, _, _ = event[3]
        self.assertEqual(error_type, ValueError)
        self.assertIs(result.wasSuccessful(), False)

    def test_picklable(self):
        result = RemoteTestResult()
        loaded_result = pickle.loads(pickle.dumps(result))
        self.assertEqual(result.events, loaded_result.events)

    def test_pickle_errors_detection(self):
        picklable_error = RuntimeError("This is fine")
        not_unpicklable_error = ExceptionThatFailsUnpickling("arg")

        result = RemoteTestResult()
        result._confirm_picklable(picklable_error)

        # The exception can be pickled but not unpickled.
        pickle.dumps(not_unpicklable_error)
        msg = "__init__() missing 1 required positional argument"
        with self.assertRaisesMessage(TypeError, msg):
            result._confirm_picklable(not_unpicklable_error)

    @unittest.skipUnless(tblib is not None, "requires tblib to be installed")
    def test_unpicklable_subtest(self):
        result = RemoteTestResult()
        subtest_test = SampleFailingSubtest(methodName="pickle_error_test")
        subtest_test.run(result=result)

        events = result.events
        subtest_event = events[1]
        assertion_error = subtest_event[3]
        self.assertEqual(str(assertion_error[1]), "expected failure")

    @unittest.skipUnless(tblib is not None, "requires tblib to be installed")
    def test_add_failing_subtests(self):
        """
        Failing subtests are added correctly using addSubTest().
        """
        # Manually run a test with failing subtests to prevent the failures
        # from affecting the actual test run.
        result = RemoteTestResult()
        subtest_test = SampleFailingSubtest(methodName="dummy_test")
        subtest_test.run(result=result)

        events = result.events
        self.assertEqual(len(events), 5)
        self.assertIs(result.wasSuccessful(), False)

        event = events[1]
        self.assertEqual(event[0], "addSubTest")
        self.assertEqual(
            str(event[2]),
            "dummy_test (test_runner.test_parallel.SampleFailingSubtest.dummy_test) "
            "(index=0)",
        )
        self.assertEqual(repr(event[3][1]), "AssertionError('0 != 1')")

        event = events[2]
        self.assertEqual(repr(event[3][1]), "AssertionError('2 != 1')")

    def test_add_duration(self):
        result = RemoteTestResult()
        result.addDuration(None, 2.3)
        self.assertEqual(result.collectedDurations, [("None", 2.3)])


class ParallelTestSuiteTest(SimpleTestCase):
    @unittest.skipUnless(tblib is not None, "requires tblib to be installed")
    def test_handle_add_error_before_first_test(self):
        dummy_subsuites = []
        pts = ParallelTestSuite(dummy_subsuites, processes=2)
        result = TestResult()
        remote_result = RemoteTestResult()
        test = SampleErrorTest(methodName="dummy_test")
        suite = TestSuite([test])
        suite.run(remote_result)
        for event in remote_result.events:
            pts.handle_event(result, tests=list(suite), event=event)

        self.assertEqual(len(result.errors), 1)
        actual_test, tb_and_details_str = result.errors[0]
        self.assertIsInstance(actual_test, _ErrorHolder)
        self.assertEqual(
            actual_test.id(), "setUpClass (test_runner.test_parallel.SampleErrorTest)"
        )
        self.assertIn("Traceback (most recent call last):", tb_and_details_str)
        self.assertIn("ValueError: woops", tb_and_details_str)

    def test_handle_add_error_during_test(self):
        dummy_subsuites = []
        pts = ParallelTestSuite(dummy_subsuites, processes=2)
        result = TestResult()
        test = TestCase()
        err = _test_error_exc_info()
        event = ("addError", 0, err)
        pts.handle_event(result, tests=[test], event=event)

        self.assertEqual(len(result.errors), 1)
        actual_test, tb_and_details_str = result.errors[0]
        self.assertIsInstance(actual_test, TestCase)
        self.assertEqual(actual_test.id(), "unittest.case.TestCase.runTest")
        self.assertIn("Traceback (most recent call last):", tb_and_details_str)
        self.assertIn("ValueError: woops", tb_and_details_str)

    def test_handle_add_failure(self):
        dummy_subsuites = []
        pts = ParallelTestSuite(dummy_subsuites, processes=2)
        result = TestResult()
        test = TestCase()
        err = _test_error_exc_info()
        event = ("addFailure", 0, err)
        pts.handle_event(result, tests=[test], event=event)

        self.assertEqual(len(result.failures), 1)
        actual_test, tb_and_details_str = result.failures[0]
        self.assertIsInstance(actual_test, TestCase)
        self.assertEqual(actual_test.id(), "unittest.case.TestCase.runTest")
        self.assertIn("Traceback (most recent call last):", tb_and_details_str)
        self.assertIn("ValueError: woops", tb_and_details_str)

    def test_handle_add_success(self):
        dummy_subsuites = []
        pts = ParallelTestSuite(dummy_subsuites, processes=2)
        result = TestResult()
        test = TestCase()
        event = ("addSuccess", 0)
        pts.handle_event(result, tests=[test], event=event)

        self.assertEqual(len(result.errors), 0)
        self.assertEqual(len(result.failures), 0)

    @unittest.skipUnless(tblib is not None, "requires tblib to be installed")
    def test_buffer_mode_reports_setupclass_failure(self):
        test = SampleErrorTest("dummy_test")
        remote_result = RemoteTestResult()
        suite = TestSuite([test])
        suite.run(remote_result)

        pts = ParallelTestSuite([suite], processes=2, buffer=True)
        pts.serialized_aliases = set()
        test_result = TestResult()
        test_result.buffer = True

        with unittest.mock.patch("multiprocessing.Pool") as mock_pool:

            def fake_next(*args, **kwargs):
                test_result.shouldStop = True
                return (0, remote_result.events)

            mock_pool.return_value.imap_unordered.return_value = unittest.mock.Mock(
                next=fake_next
            )
            pts.run(test_result)

        self.assertIn("ValueError: woops", test_result.errors[0][1])



import time

from utils_tests.test_csp import basic_config, basic_policy

from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.test import SimpleTestCase
from django.test.selenium import SeleniumTestCase
from django.test.utils import modify_settings, override_settings
from django.utils.csp import CSP

from .views import csp_reports


@override_settings(
    MIDDLEWARE=["django.middleware.csp.ContentSecurityPolicyMiddleware"],
    ROOT_URLCONF="middleware.urls",
)
class CSPMiddlewareTest(SimpleTestCase):
    @override_settings(SECURE_CSP=None, SECURE_CSP_REPORT_ONLY=None)
    def test_csp_defaults_off(self):
        response = self.client.get("/csp-base/")
        self.assertNotIn(CSP.HEADER_ENFORCE, response)
        self.assertNotIn(CSP.HEADER_REPORT_ONLY, response)

    @override_settings(SECURE_CSP=basic_config, SECURE_CSP_REPORT_ONLY=None)
    def test_csp_basic(self):
        """
        With SECURE_CSP set to a valid value, the middleware adds a
        "Content-Security-Policy" header to the response.
        """
        response = self.client.get("/csp-base/")
        self.assertEqual(response[CSP.HEADER_ENFORCE], basic_policy)
        self.assertNotIn(CSP.HEADER_REPORT_ONLY, response)

    @override_settings(SECURE_CSP={"default-src": [CSP.SELF, CSP.NONCE]})
    def test_csp_basic_with_nonce(self):
        """
        Test the nonce is added to the header and matches what is in the view.
        """
        response = self.client.get("/csp-nonce/")
        nonce = response.text
        self.assertTrue(nonce)
        self.assertEqual(
            response[CSP.HEADER_ENFORCE], f"default-src 'self' 'nonce-{nonce}'"
        )

    @override_settings(SECURE_CSP={"default-src": [CSP.SELF, CSP.NONCE]})
    def test_csp_basic_with_nonce_but_unused(self):
        """
        Test if `request.csp_nonce` is never accessed, it is not added to the
        header.
        """
        response = self.client.get("/csp-base/")
        nonce = response.text
        self.assertIsNotNone(nonce)
        self.assertEqual(response[CSP.HEADER_ENFORCE], basic_policy)

    @override_settings(SECURE_CSP=None, SECURE_CSP_REPORT_ONLY=basic_config)
    def test_csp_report_only_basic(self):
        """
        With SECURE_CSP_REPORT_ONLY set to a valid value, the middleware adds a
        "Content-Security-Policy-Report-Only" header to the response.
        """
        response = self.client.get("/csp-base/")
        self.assertEqual(response[CSP.HEADER_REPORT_ONLY], basic_policy)
        self.assertNotIn(CSP.HEADER_ENFORCE, response)

    @override_settings(
        SECURE_CSP=basic_config,
        SECURE_CSP_REPORT_ONLY=basic_config,
    )
    def test_csp_both(self):
        """
        If both SECURE_CSP and SECURE_CSP_REPORT_ONLY are set, the middleware
        adds both headers to the response.
        """
        response = self.client.get("/csp-base/")
        self.assertEqual(response[CSP.HEADER_ENFORCE], basic_policy)
        self.assertEqual(response[CSP.HEADER_REPORT_ONLY], basic_policy)

    @override_settings(
        DEBUG=True,
        SECURE_CSP=basic_config,
        SECURE_CSP_REPORT_ONLY=basic_config,
    )
    def test_csp_404_debug_view(self):
        """
        Test that the CSP headers are not added to the debug view.
        """
        response = self.client.get("/csp-404/")
        self.assertNotIn(CSP.HEADER_ENFORCE, response)
        self.assertNotIn(CSP.HEADER_REPORT_ONLY, response)

    @override_settings(
        DEBUG=True,
        SECURE_CSP=basic_config,
        SECURE_CSP_REPORT_ONLY=basic_config,
    )
    def test_csp_500_debug_view(self):
        """
        Test that the CSP headers are not added to the debug view.
        """
        with self.assertLogs("django.request", "WARNING"):
            response = self.client.get("/csp-500/")
        self.assertNotIn(CSP.HEADER_ENFORCE, response)
        self.assertNotIn(CSP.HEADER_REPORT_ONLY, response)


@override_settings(
    MIDDLEWARE=["django.middleware.csp.ContentSecurityPolicyMiddleware"],
    ROOT_URLCONF="middleware.urls",
    SECURE_CSP=basic_config,
    SECURE_CSP_REPORT_ONLY=basic_config,
)
class CSPMiddlewareWithDecoratedViewsTest(SimpleTestCase):
    def test_no_decorators(self):
        response = self.client.get("/csp-base/")
        self.assertEqual(response[CSP.HEADER_ENFORCE], basic_policy)
        self.assertEqual(response[CSP.HEADER_REPORT_ONLY], basic_policy)

    def test_csp_disabled_enforced(self):
        """
        `csp_override({})` only disables the enforced CSP header.
        """
        response = self.client.get("/csp-disabled-enforced/")
        self.assertNotIn(CSP.HEADER_ENFORCE, response)
        self.assertEqual(response[CSP.HEADER_REPORT_ONLY], basic_policy)

    def test_csp_report_only_disabled(self):
        """
        `csp_report_only_override({})` only disables the report-only header.
        """
        response = self.client.get("/csp-disabled-report-only/")
        self.assertNotIn(CSP.HEADER_REPORT_ONLY, response)
        self.assertEqual(response[CSP.HEADER_ENFORCE], basic_policy)

    def test_csp_disabled_both(self):
        """
        Using both CSP decorators with empty mappings will clear both headers.
        """
        response = self.client.get("/csp-disabled-both/")
        self.assertNotIn(CSP.HEADER_ENFORCE, response)
        self.assertNotIn(CSP.HEADER_REPORT_ONLY, response)

    def test_csp_override_enforced(self):
        """
        `csp_override` only overrides the enforced header.
        """
        response = self.client.get("/csp-override-enforced/")
        self.assertEqual(
            response[CSP.HEADER_ENFORCE], "default-src 'self'; img-src 'self' data:"
        )
        self.assertEqual(response[CSP.HEADER_REPORT_ONLY], basic_policy)

    def test_csp_report_only_override(self):
        """
        `csp_report_only_override` only overrides the report-only header.
        """
        response = self.client.get("/csp-override-report-only/")
        self.assertEqual(
            response[CSP.HEADER_REPORT_ONLY], "default-src 'self'; img-src 'self' data:"
        )
        self.assertEqual(response[CSP.HEADER_ENFORCE], basic_policy)

    def test_csp_override_both_decorator(self):
        """
        Using both CSP decorators overrides both CSP Django settings.
        """
        response = self.client.get("/csp-override-both/")
        self.assertEqual(
            response[CSP.HEADER_ENFORCE], "default-src 'self'; img-src 'self' data:"
        )
        self.assertEqual(
            response[CSP.HEADER_REPORT_ONLY], "default-src 'self'; img-src 'self' data:"
        )


@override_settings(
    ROOT_URLCONF="middleware.urls",
    SECURE_CSP_REPORT_ONLY={
        "default-src": [CSP.NONE],
        "img-src": [CSP.SELF],
        "script-src": [CSP.SELF],
        "style-src": [CSP.SELF],
        "report-uri": "/csp-report/",
    },
)
@modify_settings(
    MIDDLEWARE={"append": "django.middleware.csp.ContentSecurityPolicyMiddleware"}
)
class CSPSeleniumTestCase(SeleniumTestCase, StaticLiveServerTestCase):
    available_apps = ["middleware"]

    def setUp(self):
        self.addCleanup(csp_reports.clear)
        super().setUp()

    def test_reports_are_generated(self):
        url = self.live_server_url + "/csp-failure/"
        self.selenium.get(url)
        time.sleep(1)  # Allow time for the CSP report to be sent.
        reports = sorted(
            (r["csp-report"]["document-uri"], r["csp-report"]["violated-directive"])
            for r in csp_reports
        )
        self.assertEqual(reports, [(url, "img-src"), (url, "style-src-elem")])



from unittest.mock import patch

from django.db import NotSupportedError, connection
from django.db.models import (
    Case,
    F,
    FilteredRelation,
    OuterRef,
    Q,
    Subquery,
    TextField,
    Value,
    When,
)
from django.db.models.functions import Cast
from django.db.models.lookups import Exact
from django.test import TestCase, skipIfDBFeature, skipUnlessDBFeature

from .models import Comment, Tenant, User


class CompositePKFilterTests(TestCase):
    maxDiff = None

    @classmethod
    def setUpTestData(cls):
        cls.tenant_1 = Tenant.objects.create()
        cls.tenant_2 = Tenant.objects.create()
        cls.tenant_3 = Tenant.objects.create()
        cls.user_1 = User.objects.create(
            tenant=cls.tenant_1,
            id=1,
            email="user0001@example.com",
        )
        cls.user_2 = User.objects.create(
            tenant=cls.tenant_1,
            id=2,
            email="user0002@example.com",
        )
        cls.user_3 = User.objects.create(
            tenant=cls.tenant_2,
            id=3,
            email="user0003@example.com",
        )
        cls.user_4 = User.objects.create(
            tenant=cls.tenant_3,
            id=4,
            email="user0004@example.com",
        )
        cls.comment_1 = Comment.objects.create(id=1, user=cls.user_1)
        cls.comment_2 = Comment.objects.create(id=2, user=cls.user_1)
        cls.comment_3 = Comment.objects.create(id=3, user=cls.user_2)
        cls.comment_4 = Comment.objects.create(id=4, user=cls.user_3)
        cls.comment_5 = Comment.objects.create(id=5, user=cls.user_1)

    def test_filter_and_count_user_by_pk(self):
        test_cases = (
            ({"pk": self.user_1.pk}, 1),
            ({"pk": self.user_2.pk}, 1),
            ({"pk": self.user_3.pk}, 1),
            ({"pk": (self.tenant_1.id, self.user_1.id)}, 1),
            ({"pk": (self.tenant_1.id, self.user_2.id)}, 1),
            ({"pk": (self.tenant_2.id, self.user_3.id)}, 1),
            ({"pk": (self.tenant_1.id, self.user_3.id)}, 0),
            ({"pk": (self.tenant_2.id, self.user_1.id)}, 0),
            ({"pk": (self.tenant_2.id, self.user_2.id)}, 0),
        )

        for lookup, count in test_cases:
            with self.subTest(lookup=lookup, count=count):
                self.assertEqual(User.objects.filter(**lookup).count(), count)

    def test_rhs_pk(self):
        msg = "CompositePrimaryKey cannot be used as a lookup value."
        with self.assertRaisesMessage(ValueError, msg):
            Comment.objects.filter(text__gt=F("pk")).count()

    def test_rhs_combinable(self):
        msg = "CombinedExpression expression does not support composite primary keys."
        for expr in [F("pk") + (1, 1), (1, 1) + F("pk")]:
            with (
                self.subTest(expression=expr),
                self.assertRaisesMessage(ValueError, msg),
            ):
                Comment.objects.filter(text__gt=expr).count()

    def test_filter_comments_by_pk_gt(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        test_cases = (
            (c11, (c12, c13, c15, c24)),
            (c12, (c13, c15, c24)),
            (c13, (c15, c24)),
            (c15, (c24,)),
            (c24, ()),
        )

        for obj, objs in test_cases:
            with self.subTest(obj=obj, objs=objs):
                self.assertSequenceEqual(
                    Comment.objects.filter(pk__gt=obj.pk).order_by("pk"), objs
                )

    def test_filter_comments_by_pk_gte(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        test_cases = (
            (c11, (c11, c12, c13, c15, c24)),
            (c12, (c12, c13, c15, c24)),
            (c13, (c13, c15, c24)),
            (c15, (c15, c24)),
            (c24, (c24,)),
        )

        for obj, objs in test_cases:
            with self.subTest(obj=obj, objs=objs):
                self.assertSequenceEqual(
                    Comment.objects.filter(pk__gte=obj.pk).order_by("pk"), objs
                )

    def test_filter_comments_by_pk_lt(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        test_cases = (
            (c24, (c11, c12, c13, c15)),
            (c15, (c11, c12, c13)),
            (c13, (c11, c12)),
            (c12, (c11,)),
            (c11, ()),
        )

        for obj, objs in test_cases:
            with self.subTest(obj=obj, objs=objs):
                self.assertSequenceEqual(
                    Comment.objects.filter(pk__lt=obj.pk).order_by("pk"), objs
                )

    def test_filter_comments_by_pk_lte(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        test_cases = (
            (c24, (c11, c12, c13, c15, c24)),
            (c15, (c11, c12, c13, c15)),
            (c13, (c11, c12, c13)),
            (c12, (c11, c12)),
            (c11, (c11,)),
        )

        for obj, objs in test_cases:
            with self.subTest(obj=obj, objs=objs):
                self.assertSequenceEqual(
                    Comment.objects.filter(pk__lte=obj.pk).order_by("pk"), objs
                )

    def test_filter_comments_by_pk_in(self):
        test_cases = (
            (),
            (self.comment_1,),
            (self.comment_1, self.comment_4),
        )

        for objs in test_cases:
            with self.subTest(objs=objs):
                pks = [obj.pk for obj in objs]
                self.assertSequenceEqual(
                    Comment.objects.filter(pk__in=pks).order_by("pk"), objs
                )

    def test_filter_comments_by_pk_in_subquery(self):
        self.assertSequenceEqual(
            Comment.objects.filter(
                pk__in=Comment.objects.filter(pk=self.comment_1.pk),
            ),
            [self.comment_1],
        )
        self.assertSequenceEqual(
            Comment.objects.filter(
                pk__in=Comment.objects.filter(pk=self.comment_1.pk).values(
                    "tenant_id", "id"
                ),
            ),
            [self.comment_1],
        )
        self.comment_2.integer = self.comment_1.id
        self.comment_2.save()
        self.assertSequenceEqual(
            Comment.objects.filter(
                pk__in=Comment.objects.values("tenant_id", "integer"),
            ),
            [self.comment_1],
        )

    def test_filter_by_pk_in_subquery_invalid_selected_columns(self):
        msg = (
            "The QuerySet value for the 'in' lookup must have 2 selected "
            "fields (received 3)"
        )
        with self.assertRaisesMessage(ValueError, msg):
            Comment.objects.filter(pk__in=Comment.objects.values("pk", "text"))

    def test_filter_by_pk_in_none(self):
        with self.assertNumQueries(0):
            self.assertSequenceEqual(
                Comment.objects.filter(pk__in=[(None, 1), (1, None)]),
                [],
            )

    def test_filter_comments_by_user_and_order_by_pk_asc(self):
        self.assertSequenceEqual(
            Comment.objects.filter(user=self.user_1).order_by("pk"),
            (self.comment_1, self.comment_2, self.comment_5),
        )

    def test_filter_comments_by_user_and_order_by_pk_desc(self):
        self.assertSequenceEqual(
            Comment.objects.filter(user=self.user_1).order_by("-pk"),
            (self.comment_5, self.comment_2, self.comment_1),
        )

    def test_filter_comments_by_user_and_exclude_by_pk(self):
        self.assertSequenceEqual(
            Comment.objects.filter(user=self.user_1)
            .exclude(pk=self.comment_1.pk)
            .order_by("pk"),
            (self.comment_2, self.comment_5),
        )

    def test_filter_comments_by_user_and_contains(self):
        self.assertIs(
            Comment.objects.filter(user=self.user_1).contains(self.comment_1), True
        )

    def test_filter_query_does_not_mutate(self):
        queryset = User.objects.filter(comments__in=Comment.objects.all())
        self.assertEqual(str(queryset.query), str(queryset.query))

    def test_filter_users_by_comments_in(self):
        c1, c2, c3, c4, c5 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        u1, u2, u3 = (
            self.user_1,
            self.user_2,
            self.user_3,
        )
        test_cases = (
            ((), ()),
            ((c1,), (u1,)),
            ((c1, c2), (u1, u1)),
            ((c1, c2, c3), (u1, u1, u2)),
            ((c1, c2, c3, c4), (u1, u1, u2, u3)),
            ((c1, c2, c3, c4, c5), (u1, u1, u1, u2, u3)),
        )

        for comments, users in test_cases:
            with self.subTest(comments=comments, users=users):
                self.assertSequenceEqual(
                    User.objects.filter(comments__in=comments).order_by("pk"), users
                )

    def test_filter_users_by_comments_lt(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        u1, u2 = (
            self.user_1,
            self.user_2,
        )
        test_cases = (
            (c11, ()),
            (c12, (u1,)),
            (c13, (u1, u1)),
            (c15, (u1, u1, u2)),
            (c24, (u1, u1, u1, u2)),
        )

        for comment, users in test_cases:
            with self.subTest(comment=comment, users=users):
                self.assertSequenceEqual(
                    User.objects.filter(comments__lt=comment).order_by("pk"), users
                )

    def test_filter_users_by_comments_lte(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        u1, u2, u3 = (
            self.user_1,
            self.user_2,
            self.user_3,
        )
        test_cases = (
            (c11, (u1,)),
            (c12, (u1, u1)),
            (c13, (u1, u1, u2)),
            (c15, (u1, u1, u1, u2)),
            (c24, (u1, u1, u1, u2, u3)),
        )

        for comment, users in test_cases:
            with self.subTest(comment=comment, users=users):
                self.assertSequenceEqual(
                    User.objects.filter(comments__lte=comment).order_by("pk"), users
                )

    def test_filter_users_by_comments_gt(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        u1, u2, u3 = (
            self.user_1,
            self.user_2,
            self.user_3,
        )
        test_cases = (
            (c11, (u1, u1, u2, u3)),
            (c12, (u1, u2, u3)),
            (c13, (u1, u3)),
            (c15, (u3,)),
            (c24, ()),
        )

        for comment, users in test_cases:
            with self.subTest(comment=comment, users=users):
                self.assertSequenceEqual(
                    User.objects.filter(comments__gt=comment).order_by("pk"), users
                )

    def test_filter_users_by_comments_gte(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        u1, u2, u3 = (
            self.user_1,
            self.user_2,
            self.user_3,
        )
        test_cases = (
            (c11, (u1, u1, u1, u2, u3)),
            (c12, (u1, u1, u2, u3)),
            (c13, (u1, u2, u3)),
            (c15, (u1, u3)),
            (c24, (u3,)),
        )

        for comment, users in test_cases:
            with self.subTest(comment=comment, users=users):
                self.assertSequenceEqual(
                    User.objects.filter(comments__gte=comment).order_by("pk"), users
                )

    def test_filter_users_by_comments_exact(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )
        u1, u2, u3 = (
            self.user_1,
            self.user_2,
            self.user_3,
        )
        test_cases = (
            (c11, (u1,)),
            (c12, (u1,)),
            (c13, (u2,)),
            (c15, (u1,)),
            (c24, (u3,)),
        )

        for comment, users in test_cases:
            with self.subTest(comment=comment, users=users):
                self.assertSequenceEqual(
                    User.objects.filter(comments=comment).order_by("pk"), users
                )

    def test_filter_users_by_comments_isnull(self):
        u1, u2, u3, u4 = (
            self.user_1,
            self.user_2,
            self.user_3,
            self.user_4,
        )

        with self.subTest("comments__isnull=True"):
            self.assertSequenceEqual(
                User.objects.filter(comments__isnull=True).order_by("pk"),
                (u4,),
            )
        with self.subTest("comments__isnull=False"):
            self.assertSequenceEqual(
                User.objects.filter(comments__isnull=False).order_by("pk"),
                (u1, u1, u1, u2, u3),
            )

    def test_filter_comments_by_pk_isnull(self):
        c11, c12, c13, c24, c15 = (
            self.comment_1,
            self.comment_2,
            self.comment_3,
            self.comment_4,
            self.comment_5,
        )

        with self.subTest("pk__isnull=True"):
            self.assertSequenceEqual(
                Comment.objects.filter(pk__isnull=True).order_by("pk"),
                (),
            )
        with self.subTest("pk__isnull=False"):
            self.assertSequenceEqual(
                Comment.objects.filter(pk__isnull=False).order_by("pk"),
                (c11, c12, c13, c15, c24),
            )

    def test_filter_users_by_comments_subquery(self):
        subquery = Comment.objects.filter(id=3).only("pk")
        queryset = User.objects.filter(comments__in=subquery)
        self.assertSequenceEqual(queryset, (self.user_2,))

    def test_cannot_cast_pk(self):
        msg = "Cast expression does not support composite primary keys."
        with self.assertRaisesMessage(ValueError, msg):
            Comment.objects.filter(text__gt=Cast(F("pk"), TextField())).count()

    def test_explicit_subquery(self):
        subquery = Subquery(User.objects.values("pk"))
        self.assertEqual(User.objects.filter(pk__in=subquery).count(), 4)
        self.assertEqual(Comment.objects.filter(user__in=subquery).count(), 5)

    def test_filter_case_when(self):
        msg = "When expression does not support composite primary keys."
        with self.assertRaisesMessage(ValueError, msg):
            Comment.objects.filter(text=Case(When(text="", then="pk")))
        msg = "Case expression does not support composite primary keys."
        with self.assertRaisesMessage(ValueError, msg):
            Comment.objects.filter(text=Case(When(text="", then="text"), default="pk"))

    def test_outer_ref_pk(self):
        subquery = Subquery(Comment.objects.filter(pk=OuterRef("pk")).values("id")[:1])
        tests = [
            ("", 5),
            ("__gt", 0),
            ("__gte", 5),
            ("__lt", 0),
            ("__lte", 5),
        ]
        for lookup, expected_count in tests:
            with self.subTest(f"id{lookup}"):
                queryset = Comment.objects.filter(**{f"id{lookup}": subquery})
                self.assertEqual(queryset.count(), expected_count)

    def test_outer_ref_pk_filter_on_pk_exact(self):
        subquery = Subquery(User.objects.filter(pk=OuterRef("pk")).values("pk")[:1])
        qs = Comment.objects.filter(pk=subquery)
        self.assertEqual(qs.count(), 2)

    @skipUnlessDBFeature("supports_tuple_comparison_against_subquery")
    def test_outer_ref_pk_filter_on_pk_comparison(self):
        subquery = Subquery(User.objects.filter(pk=OuterRef("pk")).values("pk")[:1])
        tests = [
            ("gt", 0),
            ("gte", 2),
            ("lt", 0),
            ("lte", 2),
        ]
        for lookup, expected_count in tests:
            with self.subTest(f"pk__{lookup}"):
                qs = Comment.objects.filter(**{f"pk__{lookup}": subquery})
                self.assertEqual(qs.count(), expected_count)

    @skipIfDBFeature("supports_tuple_comparison_against_subquery")
    def test_outer_ref_pk_filter_on_pk_comparison_unsupported(self):
        subquery = Subquery(User.objects.filter(pk=OuterRef("pk")).values("pk")[:1])
        tests = ["gt", "gte", "lt", "lte"]
        for lookup in tests:
            with self.subTest(f"pk__{lookup}"):
                qs = Comment.objects.filter(**{f"pk__{lookup}": subquery})
                with self.assertRaisesMessage(
                    NotSupportedError,
                    f'"{lookup}" cannot be used to target composite fields '
                    "through subqueries on this backend",
                ):
                    qs.count()

    def test_unsupported_rhs(self):
        pk = Exact(F("tenant_id"), 1)
        msg = (
            "'exact' subquery lookup of 'pk' only supports OuterRef "
            "and QuerySet objects (received 'Exact')"
        )
        with self.assertRaisesMessage(ValueError, msg):
            Comment.objects.filter(pk=pk)

    @skipUnlessDBFeature("allow_sliced_subqueries_with_in")
    def test_filter_comments_by_pk_exact_subquery(self):
        self.assertSequenceEqual(
            Comment.objects.filter(
                pk=Comment.objects.filter(pk=self.comment_1.pk)[:1],
            ),
            [self.comment_1],
        )
        self.assertSequenceEqual(
            Comment.objects.filter(
                pk__in=Comment.objects.filter(pk=self.comment_1.pk).values(
                    "tenant_id", "id"
                )[:1],
            ),
            [self.comment_1],
        )
        self.comment_2.integer = self.comment_1.id
        self.comment_2.save()
        self.assertSequenceEqual(
            Comment.objects.filter(
                pk__in=Comment.objects.values("tenant_id", "integer"),
            )[:1],
            [self.comment_1],
        )

    def test_outer_ref_not_composite_pk(self):
        subquery = Comment.objects.filter(pk=OuterRef("id")).values("id")[:1]
        queryset = Comment.objects.filter(id=Subquery(subquery))

        msg = "Composite field lookups only work with composite expressions."
        with self.assertRaisesMessage(ValueError, msg):
            self.assertEqual(queryset.count(), 5)

    def test_outer_ref_in_filtered_relation(self):
        msg = (
            "This queryset contains a reference to an outer query and may only be used "
            "in a subquery."
        )
        with self.assertRaisesMessage(ValueError, msg):
            self.assertSequenceEqual(
                Tenant.objects.annotate(
                    filtered_tokens=FilteredRelation(
                        "tokens",
                        condition=Q(tokens__pk__gte=OuterRef("tokens")),
                    )
                ).filter(filtered_tokens=(1, 1)),
                [self.tenant_1],
            )

    def test_filter_by_tuple_containing_expression(self):
        pk_lookup = (self.comment_1.tenant.id, (Value(self.comment_1.id) + 1) - 1)
        for lookup in ({"pk": pk_lookup}, {"pk__in": [pk_lookup]}):
            with self.subTest(lookup=lookup):
                qs = Comment.objects.filter(**lookup)
                self.assertEqual(qs.get(), self.comment_1)


@skipUnlessDBFeature("supports_tuple_lookups")
class CompositePKFilterTupleLookupFallbackTests(CompositePKFilterTests):
    def setUp(self):
        feature_patch_1 = patch.object(
            connection.features, "supports_tuple_lookups", False
        )
        feature_patch_2 = patch.object(
            connection.features, "supports_tuple_comparison_against_subquery", False
        )
        self.enterContext(feature_patch_1)
        self.enterContext(feature_patch_2)



from django.core.paginator import AsyncPage, AsyncPaginator, Page, Paginator


class ValidAdjacentNumsPage(Page):
    def next_page_number(self):
        if not self.has_next():
            return None
        return super().next_page_number()

    def previous_page_number(self):
        if not self.has_previous():
            return None
        return super().previous_page_number()


class ValidAdjacentNumsPaginator(Paginator):
    def _get_page(self, *args, **kwargs):
        return ValidAdjacentNumsPage(*args, **kwargs)


class AsyncValidAdjacentNumsPage(AsyncPage):
    async def anext_page_number(self):
        if not await self.ahas_next():
            return None
        return await super().anext_page_number()

    async def aprevious_page_number(self):
        if not await self.ahas_previous():
            return None
        return await super().aprevious_page_number()


class AsyncValidAdjacentNumsPaginator(AsyncPaginator):
    def _get_page(self, *args, **kwargs):
        return AsyncValidAdjacentNumsPage(*args, **kwargs)



from asgiref.sync import iscoroutinefunction

from django.http import HttpRequest, HttpResponse
from django.test import SimpleTestCase
from django.views.decorators.common import no_append_slash


class NoAppendSlashTests(SimpleTestCase):
    def test_wrapped_sync_function_is_not_coroutine_function(self):
        def sync_view(request):
            return HttpResponse()

        wrapped_view = no_append_slash(sync_view)
        self.assertIs(iscoroutinefunction(wrapped_view), False)

    def test_wrapped_async_function_is_coroutine_function(self):
        async def async_view(request):
            return HttpResponse()

        wrapped_view = no_append_slash(async_view)
        self.assertIs(iscoroutinefunction(wrapped_view), True)

    def test_no_append_slash_decorator(self):
        @no_append_slash
        def sync_view(request):
            return HttpResponse()

        self.assertIs(sync_view.should_append_slash, False)
        self.assertIsInstance(sync_view(HttpRequest()), HttpResponse)

    async def test_no_append_slash_decorator_async_view(self):
        @no_append_slash
        async def async_view(request):
            return HttpResponse()

        self.assertIs(async_view.should_append_slash, False)
        self.assertIsInstance(await async_view(HttpRequest()), HttpResponse)



from django.contrib.gis.db.models.functions import (
    Area,
    Distance,
    Length,
    Perimeter,
    Transform,
    Union,
)
from django.contrib.gis.geos import GEOSGeometry, LineString, Point
from django.contrib.gis.measure import D  # alias for Distance
from django.db import NotSupportedError, connection
from django.db.models import (
    Case,
    Count,
    Exists,
    F,
    IntegerField,
    OuterRef,
    Q,
    Value,
    When,
)
from django.test import TestCase, skipIfDBFeature, skipUnlessDBFeature

from ..utils import FuncTestMixin
from .models import (
    AustraliaCity,
    CensusZipcode,
    Interstate,
    SouthTexasCity,
    SouthTexasCityFt,
    SouthTexasInterstate,
    SouthTexasZipcode,
)


class DistanceTest(TestCase):
    fixtures = ["initial"]

    def setUp(self):
        # A point we are testing distances with -- using a WGS84
        # coordinate that'll be implicitly transformed to that to
        # the coordinate system of the field, EPSG:32140 (Texas South Central
        # w/units in meters)
        self.stx_pnt = GEOSGeometry(
            "POINT (-95.370401017314293 29.704867409475465)", 4326
        )
        # Another one for Australia
        self.au_pnt = GEOSGeometry("POINT (150.791 -34.4919)", 4326)

    def get_names(self, qs):
        cities = [c.name for c in qs]
        cities.sort()
        return cities

    def test_init(self):
        """
        Test initialization of distance models.
        """
        self.assertEqual(9, SouthTexasCity.objects.count())
        self.assertEqual(9, SouthTexasCityFt.objects.count())
        self.assertEqual(11, AustraliaCity.objects.count())
        self.assertEqual(4, SouthTexasZipcode.objects.count())
        self.assertEqual(4, CensusZipcode.objects.count())
        self.assertEqual(1, Interstate.objects.count())
        self.assertEqual(1, SouthTexasInterstate.objects.count())

    @skipUnlessDBFeature("supports_dwithin_lookup")
    def test_dwithin(self):
        """
        Test the `dwithin` lookup type.
        """
        # Distances -- all should be equal (except for the
        # degree/meter pair in au_cities, that's somewhat
        # approximate).
        tx_dists = [(7000, 22965.83), D(km=7), D(mi=4.349)]
        au_dists = [(0.5, 32000), D(km=32), D(mi=19.884)]

        # Expected cities for Australia and Texas.
        tx_cities = ["Downtown Houston", "Southside Place"]
        au_cities = ["Mittagong", "Shellharbour", "Thirroul", "Wollongong"]

        # Performing distance queries on two projected coordinate systems one
        # with units in meters and the other in units of U.S. survey feet.
        for dist in tx_dists:
            if isinstance(dist, tuple):
                dist1, dist2 = dist
            else:
                dist1 = dist2 = dist
            qs1 = SouthTexasCity.objects.filter(point__dwithin=(self.stx_pnt, dist1))
            qs2 = SouthTexasCityFt.objects.filter(point__dwithin=(self.stx_pnt, dist2))
            for qs in qs1, qs2:
                with self.subTest(dist=dist, qs=qs):
                    self.assertEqual(tx_cities, self.get_names(qs))

        # With a complex geometry expression
        self.assertFalse(
            SouthTexasCity.objects.exclude(point__dwithin=(Union("point", "point"), 0))
        )

        # Now performing the `dwithin` queries on a geodetic coordinate system.
        for dist in au_dists:
            with self.subTest(dist=dist):
                type_error = isinstance(dist, D) and not connection.ops.oracle
                if isinstance(dist, tuple):
                    if connection.ops.oracle or connection.ops.spatialite:
                        # Result in meters
                        dist = dist[1]
                    else:
                        # Result in units of the field
                        dist = dist[0]

                # Creating the query set.
                qs = AustraliaCity.objects.order_by("name")
                if type_error:
                    # A ValueError should be raised on PostGIS when trying to
                    # pass Distance objects into a DWithin query using a
                    # geodetic field.
                    with self.assertRaises(ValueError):
                        AustraliaCity.objects.filter(
                            point__dwithin=(self.au_pnt, dist)
                        ).count()
                else:
                    self.assertEqual(
                        au_cities,
                        self.get_names(qs.filter(point__dwithin=(self.au_pnt, dist))),
                    )

    @skipUnlessDBFeature("supports_distances_lookups")
    def test_distance_lookups(self):
        # Retrieving the cities within a 20km 'donut' w/a 7km radius 'hole'
        # (thus, Houston and Southside place will be excluded as tested in
        # the `test02_dwithin` above).
        for model in [SouthTexasCity, SouthTexasCityFt]:
            stx_pnt = self.stx_pnt.transform(
                model._meta.get_field("point").srid, clone=True
            )
            qs = model.objects.filter(point__distance_gte=(stx_pnt, D(km=7))).filter(
                point__distance_lte=(stx_pnt, D(km=20)),
            )
            cities = self.get_names(qs)
            self.assertEqual(cities, ["Bellaire", "Pearland", "West University Place"])

        # Doing a distance query using Polygons instead of a Point.
        z = SouthTexasZipcode.objects.get(name="77005")
        qs = SouthTexasZipcode.objects.exclude(name="77005").filter(
            poly__distance_lte=(z.poly, D(m=275))
        )
        self.assertEqual(["77025", "77401"], self.get_names(qs))
        # If we add a little more distance 77002 should be included.
        qs = SouthTexasZipcode.objects.exclude(name="77005").filter(
            poly__distance_lte=(z.poly, D(m=300))
        )
        self.assertEqual(["77002", "77025", "77401"], self.get_names(qs))

    @skipUnlessDBFeature("supports_distances_lookups", "supports_distance_geodetic")
    def test_geodetic_distance_lookups(self):
        """
        Test distance lookups on geodetic coordinate systems.
        """
        # Line is from Canberra to Sydney. Query is for all other cities within
        # a 100km of that line (which should exclude only Hobart & # Adelaide).
        line = GEOSGeometry("LINESTRING(144.9630 -37.8143,151.2607 -33.8870)", 4326)
        dist_qs = AustraliaCity.objects.filter(point__distance_lte=(line, D(km=100)))
        expected_cities = [
            "Batemans Bay",
            "Canberra",
            "Hillsdale",
            "Melbourne",
            "Mittagong",
            "Shellharbour",
            "Sydney",
            "Thirroul",
            "Wollongong",
        ]
        if connection.ops.spatialite:
            # SpatiaLite is less accurate and returns 102.8km for Batemans Bay.
            expected_cities.pop(0)
        self.assertEqual(expected_cities, self.get_names(dist_qs))

        msg = "2, 3, or 4-element tuple required for 'distance_lte' lookup."
        with self.assertRaisesMessage(ValueError, msg):  # Too many params.
            len(
                AustraliaCity.objects.filter(
                    point__distance_lte=(
                        "POINT(5 23)",
                        D(km=100),
                        "spheroid",
                        "4",
                        None,
                    )
                )
            )

        with self.assertRaisesMessage(ValueError, msg):  # Too few params.
            len(AustraliaCity.objects.filter(point__distance_lte=("POINT(5 23)",)))

        msg = "For 4-element tuples the last argument must be the 'spheroid' directive."
        with self.assertRaisesMessage(ValueError, msg):
            len(
                AustraliaCity.objects.filter(
                    point__distance_lte=("POINT(5 23)", D(km=100), "spheroid", "4")
                )
            )

        # Getting all cities w/in 550 miles of Hobart.
        hobart = AustraliaCity.objects.get(name="Hobart")
        qs = AustraliaCity.objects.exclude(name="Hobart").filter(
            point__distance_lte=(hobart.point, D(mi=550))
        )
        cities = self.get_names(qs)
        self.assertEqual(cities, ["Batemans Bay", "Canberra", "Melbourne"])

        # Cities that are either really close or really far from Wollongong --
        # and using different units of distance.
        wollongong = AustraliaCity.objects.get(name="Wollongong")
        d1, d2 = D(yd=19500), D(nm=400)  # Yards (~17km) & Nautical miles.

        # Normal geodetic distance lookup (uses `distance_sphere` on PostGIS.
        gq1 = Q(point__distance_lte=(wollongong.point, d1))
        gq2 = Q(point__distance_gte=(wollongong.point, d2))
        qs1 = AustraliaCity.objects.exclude(name="Wollongong").filter(gq1 | gq2)

        # Geodetic distance lookup but telling GeoDjango to use
        # `distance_spheroid` instead (we should get the same results b/c
        # accuracy variance won't matter in this test case).
        querysets = [qs1]
        if connection.features.has_DistanceSpheroid_function:
            gq3 = Q(point__distance_lte=(wollongong.point, d1, "spheroid"))
            gq4 = Q(point__distance_gte=(wollongong.point, d2, "spheroid"))
            qs2 = AustraliaCity.objects.exclude(name="Wollongong").filter(gq3 | gq4)
            querysets.append(qs2)

        for qs in querysets:
            cities = self.get_names(qs)
            self.assertEqual(cities, ["Adelaide", "Hobart", "Shellharbour", "Thirroul"])

    @skipUnlessDBFeature("supports_distances_lookups")
    def test_distance_lookups_with_expression_rhs(self):
        stx_pnt = self.stx_pnt.transform(
            SouthTexasCity._meta.get_field("point").srid, clone=True
        )
        qs = SouthTexasCity.objects.filter(
            point__distance_lte=(stx_pnt, F("radius")),
        ).order_by("name")
        self.assertEqual(
            self.get_names(qs),
            [
                "Bellaire",
                "Downtown Houston",
                "Southside Place",
                "West University Place",
            ],
        )

        # With a combined expression
        qs = SouthTexasCity.objects.filter(
            point__distance_lte=(stx_pnt, F("radius") * 2),
        ).order_by("name")
        self.assertEqual(len(qs), 5)
        self.assertIn("Pearland", self.get_names(qs))

        # With spheroid param
        if connection.features.supports_distance_geodetic:
            hobart = AustraliaCity.objects.get(name="Hobart")
            AustraliaCity.objects.update(ref_point=hobart.point)
            for ref_point in [hobart.point, F("ref_point")]:
                qs = AustraliaCity.objects.filter(
                    point__distance_lte=(ref_point, F("radius") * 70, "spheroid"),
                ).order_by("name")
                self.assertEqual(
                    self.get_names(qs), ["Canberra", "Hobart", "Melbourne"]
                )

        # With a complex geometry expression
        self.assertFalse(
            SouthTexasCity.objects.filter(
                point__distance_gt=(Union("point", "point"), 0)
            )
        )
        self.assertEqual(
            SouthTexasCity.objects.filter(
                point__distance_lte=(Union("point", "point"), 0)
            ).count(),
            SouthTexasCity.objects.count(),
        )

    @skipUnlessDBFeature("supports_distances_lookups")
    def test_distance_annotation_group_by(self):
        stx_pnt = self.stx_pnt.transform(
            SouthTexasCity._meta.get_field("point").srid,
            clone=True,
        )
        qs = (
            SouthTexasCity.objects.annotate(
                relative_distance=Case(
                    When(point__distance_lte=(stx_pnt, D(km=20)), then=Value(20)),
                    default=Value(100),
                    output_field=IntegerField(),
                ),
            )
            .values("relative_distance")
            .annotate(count=Count("pk"))
        )
        self.assertCountEqual(
            qs,
            [
                {"relative_distance": 20, "count": 5},
                {"relative_distance": 100, "count": 4},
            ],
        )

    def test_mysql_geodetic_distance_error(self):
        if not connection.ops.mysql:
            self.skipTest("This is a MySQL-specific test.")
        msg = (
            "Only numeric values of degree units are allowed on geodetic distance "
            "queries."
        )
        with self.assertRaisesMessage(ValueError, msg):
            AustraliaCity.objects.filter(
                point__distance_lte=(Point(0, 0), D(m=100))
            ).exists()

    @skipUnlessDBFeature("supports_dwithin_lookup")
    def test_dwithin_subquery(self):
        """dwithin lookup in a subquery using OuterRef as a parameter."""
        qs = CensusZipcode.objects.annotate(
            annotated_value=Exists(
                SouthTexasCity.objects.filter(
                    point__dwithin=(OuterRef("poly"), D(m=10)),
                )
            )
        ).filter(annotated_value=True)
        self.assertEqual(self.get_names(qs), ["77002", "77025", "77401"])

    @skipUnlessDBFeature("supports_dwithin_lookup", "supports_dwithin_distance_expr")
    def test_dwithin_with_expression_rhs(self):
        # LineString of Wollongong and Adelaide coords.
        ls = LineString(((150.902, -34.4245), (138.6, -34.9258)), srid=4326)
        qs = AustraliaCity.objects.filter(
            point__dwithin=(ls, F("allowed_distance")),
        ).order_by("name")
        self.assertEqual(
            self.get_names(qs),
            ["Adelaide", "Mittagong", "Shellharbour", "Thirroul", "Wollongong"],
        )

    @skipIfDBFeature("supports_dwithin_distance_expr")
    def test_dwithin_with_expression_rhs_not_supported(self):
        ls = LineString(((150.902, -34.4245), (138.6, -34.9258)), srid=4326)
        msg = (
            "This backend does not support expressions for specifying "
            "distance in the dwithin lookup."
        )
        with self.assertRaisesMessage(NotSupportedError, msg):
            list(
                AustraliaCity.objects.filter(
                    point__dwithin=(ls, F("allowed_distance")),
                )
            )


"""
=============================
Distance functions on PostGIS
=============================

                                              | Projected Geometry | Lon/lat Geometry | Geography (4326)

ST_Distance(geom1, geom2)                     |    OK (meters)     |   :-( (degrees)  |    OK (meters)

ST_Distance(geom1, geom2, use_spheroid=False) |    N/A             |   N/A            |    OK (meters), less accurate, quick

Distance_Sphere(geom1, geom2)                 |    N/A             |   OK (meters)    |    N/A

Distance_Spheroid(geom1, geom2, spheroid)     |    N/A             |   OK (meters)    |    N/A

ST_Perimeter(geom1)                           |    OK              |   :-( (degrees)  |    OK


================================
Distance functions on SpatiaLite
================================

                                                | Projected Geometry | Lon/lat Geometry

ST_Distance(geom1, geom2)                       |    OK (meters)     |      N/A

ST_Distance(geom1, geom2, use_ellipsoid=True)   |    N/A             |      OK (meters)

ST_Distance(geom1, geom2, use_ellipsoid=False)  |    N/A             |      OK (meters), less accurate, quick

Perimeter(geom1)                                |    OK              |      :-( (degrees)

"""  # NOQA


class DistanceFunctionsTests(FuncTestMixin, TestCase):
    fixtures = ["initial"]

    @skipUnlessDBFeature("has_Area_function")
    def test_area(self):
        # Reference queries:
        # SELECT ST_Area(poly) FROM distapp_southtexaszipcode;
        area_sq_m = [
            5437908.90234375,
            10183031.4389648,
            11254471.0073242,
            9881708.91772461,
        ]
        # Tolerance has to be lower for Oracle
        tol = 2
        for i, z in enumerate(
            SouthTexasZipcode.objects.annotate(area=Area("poly")).order_by("name")
        ):
            self.assertAlmostEqual(area_sq_m[i], z.area.sq_m, tol)

    @skipUnlessDBFeature("has_Distance_function")
    def test_distance_simple(self):
        """
        Test a simple distance query, with projected coordinates and without
        transformation.
        """
        lagrange = GEOSGeometry("POINT(805066.295722839 4231496.29461335)", 32140)
        houston = (
            SouthTexasCity.objects.annotate(dist=Distance("point", lagrange))
            .order_by("id")
            .first()
        )
        tol = 2 if connection.ops.oracle else 5
        self.assertAlmostEqual(houston.dist.m, 147075.069813, tol)

    @skipUnlessDBFeature("has_Distance_function", "has_Transform_function")
    def test_distance_projected(self):
        """
        Test the `Distance` function on projected coordinate systems.
        """
        # The point for La Grange, TX
        lagrange = GEOSGeometry("POINT(-96.876369 29.905320)", 4326)
        # Reference distances in feet and in meters. Got these values from
        # using the provided raw SQL statements.
        #  SELECT ST_Distance(
        #      point,
        #      ST_Transform(
        #          ST_GeomFromText('POINT(-96.876369 29.905320)', 4326),
        #          32140
        #      )
        #  )
        #  FROM distapp_southtexascity;
        m_distances = [
            147075.069813,
            139630.198056,
            140888.552826,
            138809.684197,
            158309.246259,
            212183.594374,
            70870.188967,
            165337.758878,
            139196.085105,
        ]
        #  SELECT ST_Distance(
        #      point,
        #      ST_Transform(
        #          ST_GeomFromText('POINT(-96.876369 29.905320)', 4326),
        #          2278
        #      )
        #  )
        #  FROM distapp_southtexascityft;
        ft_distances = [
            482528.79154625,
            458103.408123001,
            462231.860397575,
            455411.438904354,
            519386.252102563,
            696139.009211594,
            232513.278304279,
            542445.630586414,
            456679.155883207,
        ]

        # Testing using different variations of parameters and using models
        # with different projected coordinate systems.
        dist1 = SouthTexasCity.objects.annotate(
            distance=Distance("point", lagrange)
        ).order_by("id")
        dist2 = SouthTexasCityFt.objects.annotate(
            distance=Distance("point", lagrange)
        ).order_by("id")
        dist_qs = [dist1, dist2]

        # Ensuring expected distances are returned for each distance queryset.
        for qs in dist_qs:
            for i, c in enumerate(qs):
                with self.subTest(c=c):
                    self.assertAlmostEqual(m_distances[i], c.distance.m, -1)
                    self.assertAlmostEqual(ft_distances[i], c.distance.survey_ft, -1)

    @skipUnlessDBFeature("has_Distance_function", "supports_distance_geodetic")
    def test_distance_geodetic(self):
        """
        Test the `Distance` function on geodetic coordinate systems.
        """
        # Testing geodetic distance calculation with a non-point geometry
        # (a LineString of Wollongong and Shellharbour coords).
        ls = LineString(((150.902, -34.4245), (150.87, -34.5789)), srid=4326)

        # Reference query:
        #  SELECT ST_distance_sphere(
        #      point,
        #      ST_GeomFromText(
        #          'LINESTRING(150.9020 -34.4245,150.8700 -34.5789)',
        #          4326
        #      )
        #  )
        #  FROM distapp_australiacity ORDER BY name;
        distances = [
            1120954.92533513,
            140575.720018241,
            640396.662906304,
            60580.9693849269,
            972807.955955075,
            568451.8357838,
            40435.4335201384,
            0,
            68272.3896586844,
            12375.0643697706,
            0,
        ]
        qs = AustraliaCity.objects.annotate(distance=Distance("point", ls)).order_by(
            "name"
        )
        for city, distance in zip(qs, distances):
            with self.subTest(city=city, distance=distance):
                # Testing equivalence to within a meter (kilometer on
                # SpatiaLite).
                tol = -3 if connection.ops.spatialite else 0
                self.assertAlmostEqual(distance, city.distance.m, tol)

    @skipUnlessDBFeature("has_Distance_function", "supports_distance_geodetic")
    def test_distance_geodetic_spheroid(self):
        tol = 2 if connection.ops.oracle else 4

        # Got the reference distances using the raw SQL statements:
        #  SELECT ST_distance_spheroid(
        #      point,
        #      ST_GeomFromText('POINT(151.231341 -33.952685)', 4326),
        #      'SPHEROID["WGS 84",6378137.0,298.257223563]'
        #  )
        #  FROM distapp_australiacity WHERE (NOT (id = 11));
        #  SELECT ST_distance_sphere(
        #      point,
        #      ST_GeomFromText('POINT(151.231341 -33.952685)', 4326)
        #  )
        #  FROM distapp_australiacity
        #  WHERE (NOT (id = 11));  st_distance_sphere
        spheroid_distances = [
            60504.0628957201,
            77023.9489850262,
            49154.8867574404,
            90847.4358768573,
            217402.811919332,
            709599.234564757,
            640011.483550888,
            7772.00667991925,
            1047861.78619339,
            1165126.55236034,
        ]
        sphere_distances = [
            60580.9693849267,
            77144.0435286473,
            49199.4415344719,
            90804.7533823494,
            217713.384600405,
            709134.127242793,
            639828.157159169,
            7786.82949717788,
            1049204.06569028,
            1162623.7238134,
        ]
        # Testing with spheroid distances first.
        hillsdale = AustraliaCity.objects.get(name="Hillsdale")
        qs = (
            AustraliaCity.objects.exclude(id=hillsdale.id)
            .annotate(distance=Distance("point", hillsdale.point, spheroid=True))
            .order_by("id")
        )
        for i, c in enumerate(qs):
            with self.subTest(c=c):
                self.assertAlmostEqual(spheroid_distances[i], c.distance.m, tol)
        if connection.ops.postgis or connection.ops.spatialite:
            # PostGIS uses sphere-only distances by default, testing these as
            # well.
            qs = (
                AustraliaCity.objects.exclude(id=hillsdale.id)
                .annotate(distance=Distance("point", hillsdale.point))
                .order_by("id")
            )
            for i, c in enumerate(qs):
                with self.subTest(c=c):
                    self.assertAlmostEqual(sphere_distances[i], c.distance.m, tol)

    @skipIfDBFeature("supports_distance_geodetic")
    @skipUnlessDBFeature("has_Distance_function")
    def test_distance_function_raw_result(self):
        distance = (
            Interstate.objects.annotate(
                d=Distance(Point(0, 0, srid=4326), Point(0, 1, srid=4326)),
            )
            .first()
            .d
        )
        self.assertEqual(distance, 1)

    @skipUnlessDBFeature("has_Distance_function")
    def test_distance_function_d_lookup(self):
        qs = Interstate.objects.annotate(
            d=Distance(Point(0, 0, srid=3857), Point(0, 1, srid=3857)),
        ).filter(d=D(m=1))
        self.assertTrue(qs.exists())

    @skipUnlessDBFeature("supports_tolerance_parameter")
    def test_distance_function_tolerance_escaping(self):
        qs = (
            Interstate.objects.annotate(
                d=Distance(
                    Point(500, 500, srid=3857),
                    Point(0, 0, srid=3857),
                    tolerance="0.05) = 1 OR 1=1 OR (1+1",
                ),
            )
            .filter(d=D(m=1))
            .values("pk")
        )
        msg = "The tolerance parameter has the wrong type"
        with self.assertRaisesMessage(TypeError, msg):
            qs.exists()

    @skipUnlessDBFeature("supports_tolerance_parameter")
    def test_distance_function_tolerance(self):
        # Tolerance is greater than distance.
        qs = (
            Interstate.objects.annotate(
                d=Distance(
                    Point(0, 0, srid=3857),
                    Point(1, 1, srid=3857),
                    tolerance=1.5,
                ),
            )
            .filter(d=0)
            .values("pk")
        )
        self.assertIs(qs.exists(), True)

    @skipIfDBFeature("supports_distance_geodetic")
    @skipUnlessDBFeature("has_Distance_function")
    def test_distance_function_raw_result_d_lookup(self):
        qs = Interstate.objects.annotate(
            d=Distance(Point(0, 0, srid=4326), Point(0, 1, srid=4326)),
        ).filter(d=D(m=1))
        msg = "Distance measure is supplied, but units are unknown for result."
        with self.assertRaisesMessage(ValueError, msg):
            list(qs)

    @skipUnlessDBFeature("has_Distance_function", "has_Transform_function")
    def test_distance_transform(self):
        """
        Test the `Distance` function used with `Transform` on a geographic
        field.
        """
        # We'll be using a Polygon (created by buffering the centroid
        # of 77005 to 100m) -- which aren't allowed in geographic distance
        # queries normally, however our field has been transformed to
        # a non-geographic system.
        z = SouthTexasZipcode.objects.get(name="77005")

        # Reference query:
        # SELECT ST_Distance(
        #   ST_Transform("distapp_censuszipcode"."poly", 32140),
        #   ST_GeomFromText('<buffer_wkt>', 32140))
        # FROM "distapp_censuszipcode";
        dists_m = [3553.30384972258, 1243.18391525602, 2186.15439472242]

        # Having our buffer in the SRID of the transformation and of the field
        # -- should get the same results. The first buffer has no need for
        # transformation SQL because it is the same SRID as what was given
        # to `transform()`. The second buffer will need to be transformed,
        # however.
        buf1 = z.poly.centroid.buffer(100)
        buf2 = buf1.transform(4269, clone=True)
        ref_zips = ["77002", "77025", "77401"]

        for buf in [buf1, buf2]:
            qs = (
                CensusZipcode.objects.exclude(name="77005")
                .annotate(distance=Distance(Transform("poly", 32140), buf))
                .order_by("name")
            )
            self.assertEqual(ref_zips, sorted(c.name for c in qs))
            for i, z in enumerate(qs):
                self.assertAlmostEqual(z.distance.m, dists_m[i], 5)

    @skipUnlessDBFeature("has_Distance_function")
    def test_distance_order_by(self):
        qs = (
            SouthTexasCity.objects.annotate(
                distance=Distance("point", Point(3, 3, srid=32140))
            )
            .order_by("distance")
            .values_list("name", flat=True)
            .filter(name__in=("San Antonio", "Pearland"))
        )
        self.assertSequenceEqual(qs, ["San Antonio", "Pearland"])

    @skipUnlessDBFeature("has_Length_function")
    def test_length(self):
        """
        Test the `Length` function.
        """
        # Reference query (should use `length_spheroid`).
        #  SELECT ST_length_spheroid(
        #      ST_GeomFromText('<wkt>', 4326)
        #      'SPHEROID["WGS 84",6378137,298.257223563,
        #        AUTHORITY["EPSG","7030"]]'
        #  );
        len_m1 = 473504.769553813
        len_m2 = 4617.668

        if connection.features.supports_length_geodetic:
            qs = Interstate.objects.annotate(length=Length("path"))
            tol = 2 if connection.ops.oracle else 3
            self.assertAlmostEqual(len_m1, qs[0].length.m, tol)
            # TODO: test with spheroid argument (True and False)
        else:
            # Does not support geodetic coordinate systems.
            with self.assertRaises(NotSupportedError):
                list(Interstate.objects.annotate(length=Length("path")))

        # Now doing length on a projected coordinate system.
        i10 = SouthTexasInterstate.objects.annotate(length=Length("path")).get(
            name="I-10"
        )
        self.assertAlmostEqual(len_m2, i10.length.m, 2)
        self.assertTrue(
            SouthTexasInterstate.objects.annotate(length=Length("path"))
            .filter(length__gt=4000)
            .exists()
        )
        # Length with an explicit geometry value.
        qs = Interstate.objects.annotate(length=Length(i10.path))
        self.assertAlmostEqual(qs.first().length.m, len_m2, 2)

    @skipUnlessDBFeature("has_Perimeter_function")
    def test_perimeter(self):
        """
        Test the `Perimeter` function.
        """
        # Reference query:
        #  SELECT ST_Perimeter(distapp_southtexaszipcode.poly)
        #  FROM distapp_southtexaszipcode;
        perim_m = [
            18404.3550889361,
            15627.2108551001,
            20632.5588368978,
            17094.5996143697,
        ]
        tol = 2 if connection.ops.oracle else 7
        qs = SouthTexasZipcode.objects.annotate(perimeter=Perimeter("poly")).order_by(
            "name"
        )
        for i, z in enumerate(qs):
            self.assertAlmostEqual(perim_m[i], z.perimeter.m, tol)

        # Running on points; should return 0.
        qs = SouthTexasCity.objects.annotate(perim=Perimeter("point"))
        for city in qs:
            self.assertEqual(0, city.perim.m)

    @skipUnlessDBFeature("has_Perimeter_function")
    def test_perimeter_geodetic(self):
        # Currently only Oracle supports calculating the perimeter on geodetic
        # geometries (without being transformed).
        qs1 = CensusZipcode.objects.annotate(perim=Perimeter("poly"))
        if connection.features.supports_perimeter_geodetic:
            self.assertAlmostEqual(qs1[0].perim.m, 18406.3818954314, 3)
        else:
            with self.assertRaises(NotSupportedError):
                list(qs1)
        # But should work fine when transformed to projected coordinates
        qs2 = CensusZipcode.objects.annotate(
            perim=Perimeter(Transform("poly", 32140))
        ).filter(name="77002")
        self.assertAlmostEqual(qs2[0].perim.m, 18404.355, 3)

    @skipUnlessDBFeature(
        "supports_null_geometries", "has_Area_function", "has_Distance_function"
    )
    def test_measurement_null_fields(self):
        """
        Test the measurement functions on fields with NULL values.
        """
        # Creating SouthTexasZipcode w/NULL value.
        SouthTexasZipcode.objects.create(name="78212")
        # Performing distance/area queries against the NULL PolygonField,
        # and ensuring the result of the operations is None.
        htown = SouthTexasCity.objects.get(name="Downtown Houston")
        z = SouthTexasZipcode.objects.annotate(
            distance=Distance("poly", htown.point), area=Area("poly")
        ).get(name="78212")
        self.assertIsNone(z.distance)
        self.assertIsNone(z.area)



from io import StringIO
from unittest import mock


class AssertFormatterFailureCaughtContext:

    def __init__(self, test, shutil_which_result="nonexistent"):
        self.stdout = StringIO()
        self.stderr = StringIO()
        self.test = test
        self.shutil_which_result = shutil_which_result

    def __enter__(self):
        self.mocker = mock.patch(
            "django.core.management.utils.shutil.which",
            return_value=self.shutil_which_result,
        )
        self.mocker.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.mocker.stop()
        self.test.assertIn("Formatters failed to launch", self.stderr.getvalue())



import logging
from unittest import mock

from django.tasks import default_task_backend, task_backends
from django.tasks.backends.base import BaseTaskBackend
from django.tasks.exceptions import InvalidTask
from django.test import SimpleTestCase, override_settings

from . import tasks as test_tasks


class CustomBackend(BaseTaskBackend):
    def __init__(self, alias, params):
        super().__init__(alias, params)
        self.prefix = self.options.get("prefix", "")

    def enqueue(self, *args, **kwargs):
        logger = logging.getLogger(__name__)
        logger.info(f"{self.prefix}Task enqueued.")


class CustomBackendNoEnqueue(BaseTaskBackend):
    pass


@override_settings(
    TASKS={
        "default": {
            "BACKEND": f"{CustomBackend.__module__}.{CustomBackend.__qualname__}",
            "OPTIONS": {"prefix": "PREFIX: "},
        },
        "no_enqueue": {
            "BACKEND": f"{CustomBackendNoEnqueue.__module__}."
            f"{CustomBackendNoEnqueue.__qualname__}",
        },
    }
)
class CustomBackendTestCase(SimpleTestCase):
    def test_using_correct_backend(self):
        self.assertEqual(default_task_backend, task_backends["default"])
        self.assertIsInstance(task_backends["default"], CustomBackend)
        self.assertEqual(default_task_backend.alias, "default")
        self.assertEqual(default_task_backend.options, {"prefix": "PREFIX: "})

    @mock.patch.multiple(CustomBackend, supports_async_task=False)
    def test_enqueue_async_task_on_non_async_backend(self):
        with self.assertRaisesMessage(
            InvalidTask, "Backend does not support async Tasks."
        ):
            default_task_backend.validate_task(test_tasks.noop_task_async)

    def test_backend_does_not_support_priority(self):
        with self.assertRaisesMessage(
            InvalidTask, "Backend does not support setting priority of tasks."
        ):
            test_tasks.noop_task.using(priority=10)

    def test_options(self):
        with self.assertLogs(__name__, level="INFO") as captured_logs:
            test_tasks.noop_task.enqueue()
        self.assertEqual(len(captured_logs.output), 1)
        self.assertIn("PREFIX: Task enqueued", captured_logs.output[0])

    def test_no_enqueue(self):
        with self.assertRaisesMessage(
            TypeError,
            "Can't instantiate abstract class CustomBackendNoEnqueue "
            "without an implementation for abstract method 'enqueue'",
        ):
            test_tasks.noop_task.using(backend="no_enqueue")



from django.http import QueryDict
from django.template import RequestContext
from django.template.base import TemplateSyntaxError
from django.test import RequestFactory, SimpleTestCase

from ..utils import setup


class QueryStringTagTests(SimpleTestCase):

    request_factory = RequestFactory()

    def assertRenderEqual(self, template_name, context, expected):
        output = self.engine.render_to_string(template_name, context)
        self.assertEqual(output, expected)

    def assertTemplateSyntaxError(self, template_name, context, expected):
        with self.assertRaisesMessage(TemplateSyntaxError, expected):
            self.engine.render_to_string(template_name, context)

    @setup({"querystring_empty_get_params": "{% querystring %}"})
    def test_querystring_empty_get_params(self):
        context = RequestContext(self.request_factory.get("/"))
        self.assertRenderEqual("querystring_empty_get_params", context, expected="?")

    @setup({"querystring_remove_all_params": "{% querystring a=None %}"})
    def test_querystring_remove_all_params(self):
        non_empty_context = RequestContext(self.request_factory.get("/?a=b"))
        empty_context = RequestContext(self.request_factory.get("/"))
        for context in [non_empty_context, empty_context]:
            with self.subTest(context=context):
                self.assertRenderEqual("querystring_remove_all_params", context, "?")

    @setup(
        {
            "querystring_remove_all_params_custom_querydict": (
                "{% querystring my_query_dict my_dict a=None %}"
            )
        }
    )
    def test_querystring_remove_all_params_custom_querydict(self):
        context = {"my_query_dict": QueryDict("a=1&b=2"), "my_dict": {"b": None}}
        self.assertRenderEqual(
            "querystring_remove_all_params_custom_querydict", context, "?"
        )

    @setup({"querystring_non_empty_get_params": "{% querystring %}"})
    def test_querystring_non_empty_get_params(self):
        request = self.request_factory.get("/", {"a": "b"})
        context = RequestContext(request)
        self.assertRenderEqual(
            "querystring_non_empty_get_params", context, expected="?a=b"
        )

    @setup({"querystring_multiple": "{% querystring %}"})
    def test_querystring_multiple(self):
        request = self.request_factory.get("/", {"x": "y", "a": "b"})
        context = RequestContext(request)
        self.assertRenderEqual("querystring_multiple", context, expected="?x=y&amp;a=b")

    @setup({"querystring_multiple_lists": "{% querystring %}"})
    def test_querystring_multiple_lists(self):
        request = self.request_factory.get("/", {"x": ["y", "z"], "a": ["b", "c"]})
        context = RequestContext(request)
        expected = "?x=y&amp;x=z&amp;a=b&amp;a=c"
        self.assertRenderEqual("querystring_multiple_lists", context, expected=expected)

    @setup({"querystring_lists_with_replacement": "{% querystring a=1 %}"})
    def test_querystring_lists_with_replacement(self):
        request = self.request_factory.get("/", {"x": ["y", "z"], "a": ["b", "c"]})
        context = RequestContext(request)
        expected = "?x=y&amp;x=z&amp;a=1"
        self.assertRenderEqual(
            "querystring_lists_with_replacement", context, expected=expected
        )

    @setup({"querystring_empty_params": "{% querystring qd %}"})
    def test_querystring_empty_params(self):
        cases = [{}, QueryDict()]
        request = self.request_factory.get("/")
        qs = "?a=b"
        request_with_qs = self.request_factory.get(f"/{qs}")
        for param in cases:
            # Empty `query_dict` and nothing on `request.GET`.
            with self.subTest(param=param):
                context = RequestContext(request, {"qd": param})
                self.assertRenderEqual(
                    "querystring_empty_params", context, expected="?"
                )
            # Empty `query_dict` and a query string in `request.GET`.
            with self.subTest(param=param, qs=qs):
                context = RequestContext(request_with_qs, {"qd": param})
                expected = "?" if param is not None else qs
                self.assertRenderEqual("querystring_empty_params", context, expected)

    @setup({"querystring_replace": "{% querystring a=1 %}"})
    def test_querystring_replace(self):
        request = self.request_factory.get("/", {"x": "y", "a": "b"})
        context = RequestContext(request)
        self.assertRenderEqual("querystring_replace", context, expected="?x=y&amp;a=1")

    @setup({"querystring_add": "{% querystring test_new='something' %}"})
    def test_querystring_add(self):
        request = self.request_factory.get("/", {"a": "b"})
        context = RequestContext(request)
        self.assertRenderEqual(
            "querystring_add", context, expected="?a=b&amp;test_new=something"
        )

    @setup({"querystring_remove": "{% querystring test=None a=1 %}"})
    def test_querystring_remove(self):
        request = self.request_factory.get("/", {"test": "value", "a": "1"})
        context = RequestContext(request)
        self.assertRenderEqual("querystring_remove", context, expected="?a=1")

    @setup({"querystring_remove_nonexistent": "{% querystring nonexistent=None a=1 %}"})
    def test_querystring_remove_nonexistent(self):
        request = self.request_factory.get("/", {"x": "y", "a": "1"})
        context = RequestContext(request)
        self.assertRenderEqual(
            "querystring_remove_nonexistent", context, expected="?x=y&amp;a=1"
        )

    @setup({"querystring_remove_dict": "{% querystring my_dict a=1 %}"})
    def test_querystring_remove_from_dict(self):
        request = self.request_factory.get("/", {"test": "value"})
        context = RequestContext(request, {"my_dict": {"test": None}})
        self.assertRenderEqual("querystring_remove_dict", context, expected="?a=1")

    @setup({"querystring_remove_querydict": "{% querystring request my_query_dict %}"})
    def test_querystring_remove_querydict(self):
        request = self.request_factory.get("/", {"x": "1"})
        my_qd = QueryDict(mutable=True)
        my_qd["x"] = None
        context = RequestContext(
            request, {"request": request.GET, "my_query_dict": my_qd}
        )
        self.assertRenderEqual("querystring_remove_querydict", context, expected="?")

    @setup(
        {"querystring_remove_querydict_many": "{% querystring request my_query_dict %}"}
    )
    def test_querystring_remove_querydict_many(self):
        request = self.request_factory.get(
            "/", {"test": ["value1", "value2"], "a": [1, 2]}
        )

        qd_none = QueryDict(mutable=True)
        qd_none["test"] = None

        qd_list_none = QueryDict(mutable=True)
        qd_list_none.setlist("test", [None, None])

        qd_empty_list = QueryDict(mutable=True)
        qd_empty_list.setlist("test", [])

        for qd in (qd_none, qd_list_none, qd_empty_list):
            with self.subTest(my_query_dict=qd):
                context = RequestContext(
                    request, {"request": request.GET, "my_query_dict": qd}
                )
                self.assertRenderEqual(
                    "querystring_remove_querydict_many",
                    context,
                    expected="?a=1&amp;a=2",
                )

    @setup({"querystring_variable": "{% querystring a=a %}"})
    def test_querystring_variable(self):
        request = self.request_factory.get("/")
        context = RequestContext(request, {"a": 1})
        self.assertRenderEqual("querystring_variable", context, expected="?a=1")

    @setup({"querystring_dict": "{% querystring my_dict %}"})
    def test_querystring_dict(self):
        context = {"my_dict": {"a": 1}}
        self.assertRenderEqual("querystring_dict", context, expected="?a=1")

    @setup({"querystring_dict_list": "{% querystring my_dict %}"})
    def test_querystring_dict_list_values(self):
        context = {"my_dict": {"a": [1, 2]}}
        self.assertRenderEqual(
            "querystring_dict_list", context, expected="?a=1&amp;a=2"
        )

    @setup(
        {
            "querystring_multiple_args_override": (
                "{% querystring my_dict my_query_dict x=3 y=None %}"
            )
        }
    )
    def test_querystring_multiple_args_override(self):
        context = {"my_dict": {"x": 0, "y": 42}, "my_query_dict": QueryDict("a=1&b=2")}
        self.assertRenderEqual(
            "querystring_multiple_args_override",
            context,
            expected="?x=3&amp;a=1&amp;b=2",
        )

    @setup({"querystring_request_get_ignored": "{% querystring my_mapping %}"})
    def test_querystring_request_get_ignored(self):
        cases = [({"y": "x"}, "?y=x"), ({}, "?")]
        request = self.request_factory.get("/", {"x": "y", "a": "b"})
        for param, expected in cases:
            with self.subTest(param=param):
                context = RequestContext(request, {"my_mapping": param})
                self.assertRenderEqual(
                    "querystring_request_get_ignored", context, expected=expected
                )

    @setup({"querystring_same_arg": "{% querystring a=1 a=2 %}"})
    def test_querystring_same_arg(self):
        msg = "'querystring' received multiple values for keyword argument 'a'"
        self.assertTemplateSyntaxError("querystring_same_arg", {}, msg)

    @setup({"querystring_non_mapping_args": "{% querystring somevar %}"})
    def test_querystring_non_mapping_args(self):
        cases = [None, 0, "", []]
        request = self.request_factory.get("/")
        msg = (
            "querystring requires mappings for positional arguments (got %r "
            "instead)."
        )
        for param in cases:
            with self.subTest(param=param):
                context = RequestContext(request, {"somevar": param})
                self.assertTemplateSyntaxError(
                    "querystring_non_mapping_args", context, msg % param
                )

    @setup({"querystring_non_string_dict_keys": "{% querystring my_dict %}"})
    def test_querystring_non_string_dict_keys(self):
        context = {"my_dict": {0: 1}}
        msg = "querystring requires strings for mapping keys (got 0 instead)."
        self.assertTemplateSyntaxError("querystring_non_string_dict_keys", context, msg)

    @setup({"querystring_list": "{% querystring a=my_list %}"})
    def test_querystring_add_list(self):
        request = self.request_factory.get("/")
        context = RequestContext(request, {"my_list": [2, 3]})
        self.assertRenderEqual("querystring_list", context, expected="?a=2&amp;a=3")

    @setup({"querystring_dict": "{% querystring a=my_dict %}"})
    def test_querystring_add_dict(self):
        request = self.request_factory.get("/")
        context = RequestContext(request, {"my_dict": {i: i * 2 for i in range(3)}})
        self.assertRenderEqual(
            "querystring_dict", context, expected="?a=0&amp;a=1&amp;a=2"
        )

    @setup({"querystring_query_dict": "{% querystring request.GET a=2 %}"})
    def test_querystring_with_explicit_query_dict(self):
        request = self.request_factory.get("/", {"a": 1})
        self.assertRenderEqual(
            "querystring_query_dict", {"request": request}, expected="?a=2"
        )

    @setup({"querystring_query_dict_no_request": "{% querystring my_query_dict a=2 %}"})
    def test_querystring_with_explicit_query_dict_and_no_request(self):
        context = {"my_query_dict": QueryDict("a=1&b=2")}
        self.assertRenderEqual(
            "querystring_query_dict_no_request", context, expected="?a=2&amp;b=2"
        )

    @setup({"querystring_no_request_no_query_dict": "{% querystring %}"})
    def test_querystring_without_request_or_explicit_query_dict(self):
        msg = "'Context' object has no attribute 'request'"
        with self.assertRaisesMessage(AttributeError, msg):
            self.engine.render_to_string("querystring_no_request_no_query_dict")



from django.template import TemplateSyntaxError
from django.template.defaulttags import ForNode
from django.test import SimpleTestCase

from ..utils import setup


class ForTagTests(SimpleTestCase):
    libraries = {"custom": "template_tests.templatetags.custom"}

    @setup({"for-tag01": "{% for val in values %}{{ val }}{% endfor %}"})
    def test_for_tag01(self):
        output = self.engine.render_to_string("for-tag01", {"values": [1, 2, 3]})
        self.assertEqual(output, "123")

    @setup({"for-tag02": "{% for val in values reversed %}{{ val }}{% endfor %}"})
    def test_for_tag02(self):
        output = self.engine.render_to_string("for-tag02", {"values": [1, 2, 3]})
        self.assertEqual(output, "321")

    @setup(
        {"for-tag-vars01": "{% for val in values %}{{ forloop.counter }}{% endfor %}"}
    )
    def test_for_tag_vars01(self):
        output = self.engine.render_to_string("for-tag-vars01", {"values": [6, 6, 6]})
        self.assertEqual(output, "123")

    @setup(
        {"for-tag-vars02": "{% for val in values %}{{ forloop.counter0 }}{% endfor %}"}
    )
    def test_for_tag_vars02(self):
        output = self.engine.render_to_string("for-tag-vars02", {"values": [6, 6, 6]})
        self.assertEqual(output, "012")

    @setup(
        {
            "for-tag-vars03": (
                "{% for val in values %}{{ forloop.revcounter }}{% endfor %}"
            )
        }
    )
    def test_for_tag_vars03(self):
        output = self.engine.render_to_string("for-tag-vars03", {"values": [6, 6, 6]})
        self.assertEqual(output, "321")

    @setup(
        {
            "for-tag-vars04": (
                "{% for val in values %}{{ forloop.revcounter0 }}{% endfor %}"
            )
        }
    )
    def test_for_tag_vars04(self):
        output = self.engine.render_to_string("for-tag-vars04", {"values": [6, 6, 6]})
        self.assertEqual(output, "210")

    @setup(
        {
            "for-tag-vars05": "{% for val in values %}"
            "{% if forloop.first %}f{% else %}x{% endif %}{% endfor %}"
        }
    )
    def test_for_tag_vars05(self):
        output = self.engine.render_to_string("for-tag-vars05", {"values": [6, 6, 6]})
        self.assertEqual(output, "fxx")

    @setup(
        {
            "for-tag-vars06": "{% for val in values %}"
            "{% if forloop.last %}l{% else %}x{% endif %}{% endfor %}"
        }
    )
    def test_for_tag_vars06(self):
        output = self.engine.render_to_string("for-tag-vars06", {"values": [6, 6, 6]})
        self.assertEqual(output, "xxl")

    @setup(
        {
            "for-tag-unpack01": (
                "{% for key,value in items %}{{ key }}:{{ value }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack01(self):
        output = self.engine.render_to_string(
            "for-tag-unpack01", {"items": (("one", 1), ("two", 2))}
        )
        self.assertEqual(output, "one:1/two:2/")

    @setup(
        {
            "for-tag-unpack03": (
                "{% for key, value in items %}{{ key }}:{{ value }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack03(self):
        output = self.engine.render_to_string(
            "for-tag-unpack03", {"items": (("one", 1), ("two", 2))}
        )
        self.assertEqual(output, "one:1/two:2/")

    @setup(
        {
            "for-tag-unpack04": (
                "{% for key , value in items %}{{ key }}:{{ value }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack04(self):
        output = self.engine.render_to_string(
            "for-tag-unpack04", {"items": (("one", 1), ("two", 2))}
        )
        self.assertEqual(output, "one:1/two:2/")

    @setup(
        {
            "for-tag-unpack05": (
                "{% for key ,value in items %}{{ key }}:{{ value }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack05(self):
        output = self.engine.render_to_string(
            "for-tag-unpack05", {"items": (("one", 1), ("two", 2))}
        )
        self.assertEqual(output, "one:1/two:2/")

    @setup(
        {
            "for-tag-unpack06": (
                "{% for key value in items %}{{ key }}:{{ value }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack06(self):
        msg = "'for' tag received an invalid argument: for key value in items"
        with self.assertRaisesMessage(TemplateSyntaxError, msg):
            self.engine.render_to_string(
                "for-tag-unpack06", {"items": (("one", 1), ("two", 2))}
            )

    @setup(
        {
            "for-tag-unpack07": (
                "{% for key,,value in items %}{{ key }}:{{ value }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack07(self):
        msg = "'for' tag received an invalid argument: for key,,value in items"
        with self.assertRaisesMessage(TemplateSyntaxError, msg):
            self.engine.render_to_string(
                "for-tag-unpack07", {"items": (("one", 1), ("two", 2))}
            )

    @setup(
        {
            "for-tag-unpack08": (
                "{% for key,value, in items %}{{ key }}:{{ value }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack08(self):
        msg = "'for' tag received an invalid argument: for key,value, in items"
        with self.assertRaisesMessage(TemplateSyntaxError, msg):
            self.engine.render_to_string(
                "for-tag-unpack08", {"items": (("one", 1), ("two", 2))}
            )

    @setup({"double-quote": '{% for "k" in items %}{{ "k" }}/{% endfor %}'})
    def test_unpack_double_quote(self):
        msg = """'for' tag received an invalid argument: for "k" in items"""
        with self.assertRaisesMessage(TemplateSyntaxError, msg):
            self.engine.render_to_string("double-quote", {"items": (1, 2)})

    @setup({"single-quote": "{% for 'k' in items %}{{ k }}/{% endfor %}"})
    def test_unpack_single_quote(self):
        msg = """'for' tag received an invalid argument: for 'k' in items"""
        with self.assertRaisesMessage(TemplateSyntaxError, msg):
            self.engine.render_to_string("single-quote", {"items": (1, 2)})

    @setup({"vertical-bar": "{% for k|upper in items %}{{ k|upper }}/{% endfor %}"})
    def test_unpack_vertical_bar(self):
        msg = "'for' tag received an invalid argument: for k|upper in items"
        with self.assertRaisesMessage(TemplateSyntaxError, msg):
            self.engine.render_to_string("vertical-bar", {"items": (1, 2)})

    @setup(
        {
            "for-tag-unpack09": (
                "{% for val in items %}{{ val.0 }}:{{ val.1 }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack09(self):
        """
        A single loopvar doesn't truncate the list in val.
        """
        output = self.engine.render_to_string(
            "for-tag-unpack09", {"items": (("one", 1), ("two", 2))}
        )
        self.assertEqual(output, "one:1/two:2/")

    @setup(
        {
            "for-tag-unpack13": (
                "{% for x,y,z in items %}{{ x }}:{{ y }},{{ z }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack13(self):
        output = self.engine.render_to_string(
            "for-tag-unpack13", {"items": (("one", 1, "carrot"), ("two", 2, "cheese"))}
        )
        if self.engine.string_if_invalid:
            self.assertEqual(output, "one:1,carrot/two:2,cheese/")
        else:
            self.assertEqual(output, "one:1,carrot/two:2,cheese/")

    @setup(
        {
            "for-tag-empty01": (
                "{% for val in values %}{{ val }}{% empty %}empty text{% endfor %}"
            )
        }
    )
    def test_for_tag_empty01(self):
        output = self.engine.render_to_string("for-tag-empty01", {"values": [1, 2, 3]})
        self.assertEqual(output, "123")

    @setup(
        {
            "for-tag-empty02": (
                "{% for val in values %}{{ val }}{% empty %}values array empty"
                "{% endfor %}"
            )
        }
    )
    def test_for_tag_empty02(self):
        output = self.engine.render_to_string("for-tag-empty02", {"values": []})
        self.assertEqual(output, "values array empty")

    @setup(
        {
            "for-tag-empty03": "{% for val in values %}"
            "{{ val }}{% empty %}values array not found{% endfor %}"
        }
    )
    def test_for_tag_empty03(self):
        output = self.engine.render_to_string("for-tag-empty03")
        self.assertEqual(output, "values array not found")

    @setup(
        {
            "for-tag-filter-ws": (
                "{% load custom %}{% for x in s|noop:'x y' %}{{ x }}{% endfor %}"
            )
        }
    )
    def test_for_tag_filter_ws(self):
        """
        #19882
        """
        output = self.engine.render_to_string("for-tag-filter-ws", {"s": "abc"})
        self.assertEqual(output, "abc")

    @setup(
        {"for-tag-unpack-strs": "{% for x,y in items %}{{ x }}:{{ y }}/{% endfor %}"}
    )
    def test_for_tag_unpack_strs(self):
        output = self.engine.render_to_string(
            "for-tag-unpack-strs", {"items": ("ab", "ac")}
        )
        self.assertEqual(output, "a:b/a:c/")

    @setup({"for-tag-unpack10": "{% for x,y in items %}{{ x }}:{{ y }}/{% endfor %}"})
    def test_for_tag_unpack10(self):
        with self.assertRaisesMessage(
            ValueError, "Need 2 values to unpack in for loop; got 3."
        ):
            self.engine.render_to_string(
                "for-tag-unpack10",
                {"items": (("one", 1, "carrot"), ("two", 2, "orange"))},
            )

    @setup(
        {
            "for-tag-unpack11": (
                "{% for x,y,z in items %}{{ x }}:{{ y }},{{ z }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack11(self):
        with self.assertRaisesMessage(
            ValueError, "Need 3 values to unpack in for loop; got 2."
        ):
            self.engine.render_to_string(
                "for-tag-unpack11",
                {"items": (("one", 1), ("two", 2))},
            )

    @setup(
        {
            "for-tag-unpack12": (
                "{% for x,y,z in items %}{{ x }}:{{ y }},{{ z }}/{% endfor %}"
            )
        }
    )
    def test_for_tag_unpack12(self):
        with self.assertRaisesMessage(
            ValueError, "Need 3 values to unpack in for loop; got 2."
        ):
            self.engine.render_to_string(
                "for-tag-unpack12", {"items": (("one", 1, "carrot"), ("two", 2))}
            )

    @setup({"for-tag-unpack14": "{% for x,y in items %}{{ x }}:{{ y }}/{% endfor %}"})
    def test_for_tag_unpack14(self):
        with self.assertRaisesMessage(
            ValueError, "Need 2 values to unpack in for loop; got 1."
        ):
            self.engine.render_to_string("for-tag-unpack14", {"items": (1, 2)})

    @setup(
        {
            "main": '{% with alpha=alpha.values %}{% include "base" %}{% endwith %}_'
            '{% with alpha=alpha.extra %}{% include "base" %}{% endwith %}',
            "base": "{% for x, y in alpha %}{{ x }}:{{ y }},{% endfor %}",
        }
    )
    def test_for_tag_context(self):
        """
        ForNode.render() pops the values it pushes to the context (#28001).
        """
        output = self.engine.render_to_string(
            "main",
            {
                "alpha": {
                    "values": [("two", 2), ("four", 4)],
                    "extra": [("six", 6), ("eight", 8)],
                },
            },
        )
        self.assertEqual(output, "two:2,four:4,_six:6,eight:8,")

    @setup({"invalid_for_loop": "{% for x items %}{{ x }}{% endfor %}"})
    def test_invalid_arg(self):
        msg = "'for' statements should have at least four words: for x items"
        with self.assertRaisesMessage(TemplateSyntaxError, msg):
            self.engine.render_to_string("invalid_for_loop", {"items": (1, 2)})

    @setup({"invalid_for_loop": "{% for x from items %}{{ x }}{% endfor %}"})
    def test_invalid_in_keyword(self):
        msg = "'for' statements should use the format 'for x in y': for x from items"
        with self.assertRaisesMessage(TemplateSyntaxError, msg):
            self.engine.render_to_string("invalid_for_loop", {"items": (1, 2)})

    @setup(
        {
            "forloop-length": "{% for val in values %}{{ forloop.length }}{% endfor %}",
            "forloop-length-reversed": "{% for val in values reversed %}"
            "{{ forloop.length }}{% endfor %}",
        }
    )
    def test_forloop_length(self):
        cases = [
            ([1, 2, 3], "333"),
            ([1, 2, 3, 4, 5, 6], "666666"),
            ([], ""),
        ]
        for values, expected_output in cases:
            for template in ["forloop-length", "forloop-length-reversed"]:
                with self.subTest(expected_output=expected_output, template=template):
                    output = self.engine.render_to_string(template, {"values": values})
                    self.assertEqual(output, expected_output)


class ForNodeTests(SimpleTestCase):
    def test_repr(self):
        node = ForNode(
            "x",
            "sequence",
            is_reversed=True,
            nodelist_loop=["val"],
            nodelist_empty=["val2"],
        )
        self.assertEqual(
            repr(node), "<ForNode: for x in sequence, tail_len: 1 reversed>"
        )



import logging
import unittest
from io import StringIO
from time import time
from unittest import mock

from django.db import DEFAULT_DB_ALIAS, connection, connections
from django.test import TestCase
from django.test.runner import DiscoverRunner, QueryFormatter

from .models import Person

logger = logging.getLogger(__name__)


class QueryFormatterTests(unittest.TestCase):

    def setUp(self):
        super().setUp()
        self.format_sql_calls = []

    def new_format_sql(self, sql):
        # Use time() to introduce some uniqueness.
        formatted = "Formatted! %s at %s" % (sql.upper(), time())
        self.format_sql_calls.append({sql: formatted})
        return formatted

    def make_handler(self, **formatter_kwargs):
        formatter = QueryFormatter(**formatter_kwargs)

        handler = logging.StreamHandler(StringIO())
        handler.setLevel(logging.DEBUG)
        handler.setFormatter(formatter)

        original_level = logger.getEffectiveLevel()
        logger.setLevel(logging.DEBUG)
        self.addCleanup(logger.setLevel, original_level)
        logger.addHandler(handler)
        self.addCleanup(logger.removeHandler, handler)

        return handler

    def do_log(self, msg, *logger_args, alias=DEFAULT_DB_ALIAS, extra=None):
        if extra is None:
            extra = {}
        if alias and "alias" not in extra:
            extra["alias"] = alias
        # Patch connection's format_debug_sql to ensure it was properly called.
        with mock.patch.object(
            connections[alias].ops, "format_debug_sql", side_effect=self.new_format_sql
        ):
            logger.info(msg, *logger_args, extra=extra)

    def assertLogRecord(self, handler, expected):
        handler.stream.seek(0)
        self.assertEqual(handler.stream.read().strip(), expected)

    def assertSQLFormatted(self, handler, sql, total_calls=1):
        self.assertEqual(len(self.format_sql_calls), total_calls)
        formatted_sql = self.format_sql_calls[0][sql]
        expected = f"=> Executing query duration=3.142 sql={formatted_sql}"
        self.assertLogRecord(handler, expected)

    def test_formats_sql_bracket_format_style(self):
        handler = self.make_handler(
            fmt="{message} duration={duration:.3f} sql={sql}", style="{"
        )
        msg = "=> Executing query"
        sql = "select * from foo"

        self.do_log(msg, extra={"sql": sql, "duration": 3.1416})
        self.assertSQLFormatted(handler, sql)

    def test_formats_sql_named_fmt_format_style(self):
        handler = self.make_handler(
            fmt="%(message)s duration=%(duration).3f sql=%(sql)s"
        )
        msg = "=> Executing query"
        sql = "select * from foo"

        self.do_log(msg, extra={"sql": sql, "duration": 3.1416})
        self.assertSQLFormatted(handler, sql)

    def test_formats_sql_named_percent_format_style(self):
        handler = self.make_handler()
        msg = "=> Executing query duration=%(duration).3f sql=%(sql)s"
        sql = "select * from foo"

        self.do_log(msg, {"duration": 3.1416, "sql": sql})
        self.assertSQLFormatted(handler, sql)

    def test_formats_sql_default_percent_format_style(self):
        handler = self.make_handler()
        msg = "=> Executing query duration=%.3f sql=%s"
        sql = "select * from foo"

        self.do_log(msg, 3.1416, sql)
        self.assertSQLFormatted(handler, sql)

    def test_formats_sql_multiple_matching_sql(self):
        handler = self.make_handler()
        msg = "=> Executing query duration=%.3f sql=%s"
        sql = "select * from foo"

        self.do_log(msg, 3.1416, sql, extra={"duration": 3.1416, "sql": sql})
        self.assertSQLFormatted(handler, sql)

    def test_formats_sql_multiple_non_matching_sql(self):
        handler = self.make_handler()
        msg = "=> Executing query duration=%.3f sql=%s"
        sql1 = "select * from foo"
        sql2 = "select * from other"

        self.do_log(msg, 3.1416, sql1, extra={"duration": 3.1416, "sql": sql2})
        self.assertSQLFormatted(handler, sql1, total_calls=2)
        # Second format call is triggered since the sql are different.
        self.assertEqual(list(self.format_sql_calls[1].keys()), [sql2])

    def test_log_record_no_args(self):
        handler = self.make_handler()
        msg = "=> Executing query no args"

        self.do_log(msg)
        self.assertEqual(self.format_sql_calls, [])
        self.assertLogRecord(handler, msg)

    def test_log_record_not_enough_args(self):
        handler = self.make_handler()
        msg = "=> Executing query one args %r"
        args = "not formatted"

        self.do_log(msg, args)
        self.assertEqual(self.format_sql_calls, [])
        self.assertLogRecord(handler, msg % args)

    def test_log_record_not_key_in_dict_args(self):
        handler = self.make_handler()
        msg = "=> Executing query missing sql key %(foo)r"
        args = {"foo": "bar"}

        self.do_log(msg, args)
        self.assertEqual(self.format_sql_calls, [])
        self.assertLogRecord(handler, msg % args)

    def test_log_record_no_alias(self):
        handler = self.make_handler()
        msg = "=> Executing query duration=%.3f sql=%s"
        args = (3.1416, "select * from foo")

        self.do_log(msg, *args, extra={"alias": "does not exist"})
        self.assertEqual(self.format_sql_calls, [])
        self.assertLogRecord(handler, msg % args)

    def test_log_record_sql_arg_none(self):
        handler = self.make_handler()
        msg = "=> Executing query duration=%.3f sql=%s"
        args = (3.1416, None)

        self.do_log(msg, *args)
        self.assertEqual(self.format_sql_calls, [])
        self.assertLogRecord(handler, msg % args)

    def test_log_record_sql_key_none(self):
        handler = self.make_handler()
        msg = "=> Executing query duration=%(duration).3f sql=%(sql)s"
        args = {"duration": 3.1416, "sql": None}

        self.do_log(msg, args)
        self.assertEqual(self.format_sql_calls, [])
        self.assertLogRecord(handler, msg % args)

    def test_log_record_sql_extra_none(self):
        handler = self.make_handler(
            fmt="{message} duration={duration:.3f} sql={sql}", style="{"
        )
        msg = "=> Executing query"

        self.do_log(msg, extra={"sql": None, "duration": 3.1416})
        self.assertEqual(self.format_sql_calls, [])
        self.assertLogRecord(handler, f"{msg} duration=3.142 sql=None")


@unittest.skipUnless(
    connection.vendor == "sqlite", "Only run on sqlite so we can check output SQL."
)
class TestDebugSQL(unittest.TestCase):
    class PassingTest(TestCase):
        def runTest(self):
            Person.objects.filter(first_name="pass").count()

    class FailingTest(TestCase):
        def runTest(self):
            Person.objects.filter(first_name="fail").count()
            self.fail()

    class ErrorTest(TestCase):
        def runTest(self):
            Person.objects.filter(first_name="error").count()
            raise Exception

    class ErrorSetUpTestDataTest(TestCase):
        @classmethod
        def setUpTestData(cls):
            raise Exception

        def runTest(self):
            pass

    class PassingSubTest(TestCase):
        def runTest(self):
            with self.subTest():
                Person.objects.filter(first_name="subtest-pass").count()

    class FailingSubTest(TestCase):
        def runTest(self):
            with self.subTest():
                Person.objects.filter(first_name="subtest-fail").count()
                self.fail()

    class ErrorSubTest(TestCase):
        def runTest(self):
            with self.subTest():
                Person.objects.filter(first_name="subtest-error").count()
                raise Exception

    def _test_output(self, verbosity):
        runner = DiscoverRunner(debug_sql=True, verbosity=0)
        suite = runner.test_suite()
        suite.addTest(self.FailingTest())
        suite.addTest(self.ErrorTest())
        suite.addTest(self.PassingTest())
        suite.addTest(self.PassingSubTest())
        suite.addTest(self.FailingSubTest())
        suite.addTest(self.ErrorSubTest())
        old_config = runner.setup_databases()
        stream = StringIO()
        resultclass = runner.get_resultclass()
        runner.test_runner(
            verbosity=verbosity,
            stream=stream,
            resultclass=resultclass,
        ).run(suite)
        runner.teardown_databases(old_config)

        return stream.getvalue()

    def test_output_normal(self):
        full_output = self._test_output(1)
        for output in self.expected_outputs:
            self.assertIn(output, full_output)
        for output in self.verbose_expected_outputs:
            self.assertNotIn(output, full_output)

    def test_output_verbose(self):
        full_output = self._test_output(2)
        for output in self.expected_outputs:
            self.assertIn(output, full_output)
        for output in self.verbose_expected_outputs:
            self.assertIn(output, full_output)

    expected_outputs = [
        (
            """SELECT COUNT(*) AS "__count"\n"""
            """FROM "test_runner_person"\n"""
            """WHERE "test_runner_person"."first_name" = 'error'; """
            """args=('error',); alias=default"""
        ),
        (
            """SELECT COUNT(*) AS "__count"\n"""
            """FROM "test_runner_person"\n"""
            """WHERE "test_runner_person"."first_name" = 'fail'; """
            """args=('fail',); alias=default"""
        ),
        (
            """SELECT COUNT(*) AS "__count"\n"""
            """FROM "test_runner_person"\n"""
            """WHERE "test_runner_person"."first_name" = 'subtest-error'; """
            """args=('subtest-error',); alias=default"""
        ),
        (
            """SELECT COUNT(*) AS "__count"\n"""
            """FROM "test_runner_person"\n"""
            """WHERE "test_runner_person"."first_name" = 'subtest-fail'; """
            """args=('subtest-fail',); alias=default"""
        ),
    ]

    test_class_path = "test_runner.test_debug_sql.TestDebugSQL"
    verbose_expected_outputs = [
        f"runTest ({test_class_path}.FailingTest.runTest) ... FAIL",
        f"runTest ({test_class_path}.ErrorTest.runTest) ... ERROR",
        f"runTest ({test_class_path}.PassingTest.runTest) ... ok",
        # If there are errors/failures in subtests but not in test itself,
        # the status is not written. That behavior comes from Python.
        f"runTest ({test_class_path}.FailingSubTest.runTest) ...",
        f"runTest ({test_class_path}.ErrorSubTest.runTest) ...",
        (
            """SELECT COUNT(*) AS "__count"\n"""
            """FROM "test_runner_person"\nWHERE """
            """"test_runner_person"."first_name" = 'pass'; """
            """args=('pass',); alias=default"""
        ),
        (
            """SELECT COUNT(*) AS "__count"\n"""
            """FROM "test_runner_person"\nWHERE """
            """"test_runner_person"."first_name" = 'subtest-pass'; """
            """args=('subtest-pass',); alias=default"""
        ),
    ]

    def test_setupclass_exception(self):
        runner = DiscoverRunner(debug_sql=True, verbosity=0)
        suite = runner.test_suite()
        suite.addTest(self.ErrorSetUpTestDataTest())
        old_config = runner.setup_databases()
        stream = StringIO()
        runner.test_runner(
            verbosity=0,
            stream=stream,
            resultclass=runner.get_resultclass(),
        ).run(suite)
        runner.teardown_databases(old_config)
        output = stream.getvalue()
        self.assertIn(
            "ERROR: setUpClass "
            "(test_runner.test_debug_sql.TestDebugSQL.ErrorSetUpTestDataTest)",
            output,
        )



from django.conf import settings
from django.utils.csp import CSP, LazyNonce, build_policy
from django.utils.deprecation import MiddlewareMixin


def get_nonce(request):
    return getattr(request, "_csp_nonce", None)


class ContentSecurityPolicyMiddleware(MiddlewareMixin):
    def process_request(self, request):
        request._csp_nonce = LazyNonce()

    def process_response(self, request, response):
        nonce = get_nonce(request)

        sentinel = object()
        if (csp_config := getattr(response, "_csp_config", sentinel)) is sentinel:
            csp_config = settings.SECURE_CSP
        if (csp_ro_config := getattr(response, "_csp_ro_config", sentinel)) is sentinel:
            csp_ro_config = settings.SECURE_CSP_REPORT_ONLY

        for header, config in [
            (CSP.HEADER_ENFORCE, csp_config),
            (CSP.HEADER_REPORT_ONLY, csp_ro_config),
        ]:
            # If headers are already set on the response, don't overwrite them.
            # This allows for views to set their own CSP headers as needed.
            # An empty config means CSP headers are not added to the response.
            if config and header not in response:
                response.headers[str(header)] = build_policy(config, nonce)

        return response
